;Assembly options...
TEST_SEED    = 0   ;1 = check the random generator...uses next constant as the test seed
SEED         = $0F ;seed value $0F should produce a game that only has the bat & magnet present
FLIP_DRAGONS = 1   ;1 = have dragons face the player when chasing
EDGE_PANELS  = 1   ;1 = place panels closer to the screen border
GFX_DELIMITER = %10111101 ;constant, pattern to use at end of bitmaps...must be unique/unused


      processor 6502

;2600's hardware equates...
VSYNC   =  $00
VBLANK  =  $01
WSYNC   =  $02
RSYNC   =  $03
NUSIZ0  =  $04
NUSIZ1  =  $05
COLUP0  =  $06
COLUP1  =  $07
COLUPF  =  $08
COLUBK  =  $09
CTRLPF  =  $0A
REFP0   =  $0B
REFP1   =  $0C
PF0     =  $0D
PF1     =  $0E
PF2     =  $0F
RESP0   =  $10
AUDC0   =  $15
AUDC1   =  $16
AUDF0   =  $17
AUDF1   =  $18
AUDV0   =  $19
AUDV1   =  $1A
GRP0    =  $1B
GRP1    =  $1C
ENAM0   =  $1D
ENAM1   =  $1E
ENABL   =  $1F
HMP0    =  $20
VDELP1  =  $26
HMOVE   =  $2A
HMCLR   =  $2B
CXCLR   =  $2C
CXP0FB  =  $32
CXP1FB  =  $33
CXM0FB  =  $34
CXM1FB  =  $35
CXBLPF  =  $36
CXPPMM  =  $37
INPT4   =  $3C
SWCHA   =  $0280
SWCHB   =  $0282
INTIM   =  $0284
TIM64T  =  $0296

       SEG.U variables
       ORG $80

;game engine variables...
Obj1Lo        DS 2
Obj1Hi        = Obj1Lo+1
Obj2Lo        DS 2
Obj2Hi        = Obj2Lo+1
Obj1Y         DS 1
Obj2Y         DS 1
PlayerRoom    DS 3
PlayerX       = PlayerRoom+1
PlayerY       = PlayerRoom+2
CurrentObject DS 4
CurrentStick  DS 1 ;upper nybble = stick, bit2 = B&W toggle, bit 0/1,3 = console switch
LastObj       DS 1 ;pointer to the list of printable objects
ObjDir        DS 1
ObjAddress    DS 1
ObjLst        DS 2
Delta2        DS 1
Delta1        DS 1
ObjCr         DS 1
Level         DS 1 ;Bit0 = console type, bit3-1 = game selection, upper nybble = game sound
Control       DS 1 ;bit7=inactive game, bits6-0 = room # backup (for printing)
NoteCnt       DS 1
Index         DS 1
PrevRoom      DS 3
PrevX         = PrevRoom+1
PrevY         = PrevRoom+2
LoCnt         DS 1
HiCnt         DS 1
ObjectNumber  DS 1
Temp          DS 1
CarriedObj    DS 3
CarriedX      =  CarriedObj+1
CarriedY      =  CarriedObj+2

;Object variables...
SurroundR     DS 3 ;(3 bytes)
DotR          DS 3 ;(3 bytes) NOTE: should be the first movable object in RAM
;NOTE: all dragons must be consecutive
RDragonR      DS 5 ;(5 bytes)
YDragonR      DS 5 ;(5 bytes)
GDragonR      DS 5 ;(5 bytes)
MagnetR       DS 3 ;(3 bytes)
SwordR        DS 3 ;(3 bytes)
ChalliseR     DS 3 ;(3 bytes)
BridgeR       DS 3 ;(3 bytes)
;NOTE: all keys must be consecutive
YKeyR         DS 3 ;(3 bytes)
WKeyR         DS 3 ;(3 bytes)
BKeyR         DS 3 ;(3 bytes)
;NOTE: all gates must be consecutive
YGateR        DS 1
WGateR        DS 1
BGateR        DS 1
BatR          DS 7 ;(7 bytes)


;-------------------------------------------------------------------------------
;Ram locations D8-FB free (FC/FF used by the stack...subroutines nested 2 levels)
;-------------------------------------------------------------------------------


;Shared ram locations...
;NOTE: Temp, Obj2X, & ObjAddress may be possible to use as color pointer in kernel?
ScanLineCnt   =  ObjDir
RoomDefIndex  =  Index
RoomLo        =  ObjLst
RoomHi        =  ObjLst+1
PlayerYadj    =  Delta2
PixelCount    =  Delta1
NullR         =  BridgeR
Sound         =  Level       ;upper nybble of the Level variable
ObjX          =  Delta2      ;used only during object setup
MagSave       =  Delta1      ;used for saving/restoring magnet Y position

;Constants...
;Object numbers...these are used to calculate the offsets of the object table
SurroundNumber =  SurroundData - Objects ;($00)
YGateNumber    =  YGateData - Objects    ;($01)
WGateNumber    =  WGateData - Objects    ;($02)
BGateNumber    =  BGateData - Objects    ;($03)
AuthorNumber   =  AuthorData - Objects   ;($04)
NumberNumber   =  NumberData - Objects   ;($05)
RDragonNumber  =  RDragonData - Objects  ;($06)
YDragonNumber  =  YDragonData - Objects  ;($07)
GDragonNumber  =  GDragonData - Objects  ;($08)
SwordNumber    =  SwordData - Objects    ;($09)
BridgeNumber   =  BridgeData - Objects   ;($0A)
YKeyNumber     =  YKeyData - Objects     ;($0B)
WKeyNumber     =  WKeyData - Objects     ;($0C)
BKeyNumber     =  BKeyData - Objects     ;($0D)
BatNumber      =  BatData - Objects      ;($0E)
DotNumber      =  DotData - Objects      ;($0F)
ChalliseNumber =  ChalliseData - Objects ;($10)
MagnetNumber   =  MagnetData - Objects   ;($11)
NullNumber     =  NullData - Objects     ;($12)

;RD0 used to calculate map direction value when including differences
RD0            =  RoomDiffs - $80

;the number of castle gates...
Gates          =  $02 ;# of castle gates - 1

;Winning game screen (where to bring the challise)
WinRoom        =  $12 ;(Yellow Castle's room number)

;Game setup...
ByteNum        =  Game2Objects - (Game1Objects+1) ;number of bytes in each "fill" table
RndNum         =  Loc_4 - (Loc_1 + 3) ; offset value for the room bounds table
RndMax         =  $1F ;Mask for the upper bounds of the random seed

;height of magnet object (used to adjust attracted object)
MAGNET_HEIGHT = GfxMagnetEnd - GfxMagnet

       SEG rom code
       ORG  $1000

START1:
       LDA    $FFF9               ;4 select 2nd bank
       JMP    StartGame           ;3 (filler)

KernelDone:
       LDA    $FFF9               ;4 72 select 2nd bank
;63 from other bank...
       LDY    #$00                ;2 65 Set initial room definition index
       LAX    (RoomLo),Y          ;5 70 Get first room definition byte (use LAX to preserve)
       STA    ENAM0               ;3 73 Enable left hand wall(if wanted-missle00)
       STA    WSYNC               ;3 00 Wait for horizontal Blank
       STY    VBLANK              ;3 03 Clear any Vertical Blank
       CLC                        ;2 05
       BCC    Print_Top_Line      ;2 08 always branch

No_Playfield_Update: ;(67)
;9 cycles remain here if WSYNC removed
       STA    WSYNC               ;3 00 waste
       STX    GRP0                ;3 03 Display Player00 definition byte (if Wanted)
;NOTE: this part wastes 52 cycles...all registers are OK to use. Carry should be clear on exit.
       STA    $2E                 ;3 06 (waste)
       STA    $2E                 ;3 09 (waste)
       LDY    #$09                ;2 11
WasteLoop:
       DEY                        ;2 13/18/23/28/33/38/43/48/53
       BNE    WasteLoop           ;2 16/21/26/31/36/41/46/51/55
       CLC                        ;2 57
       BCC    PrintPlayer01       ;2 60 always branch

Enable_Ball: ;75
       LDA    #$02                ;2 01 Enable Ball Graphic
Disable_Ball: ;01
       STA    ENABL               ;3 04 Enable Ball (If Wanted)...waste remaining cycle
       LDY    #$00                ;2 06 Clear Y index
       STX    ScanLineCnt         ;3 09 Store the scan line
       TXA                        ;2 11
       SEC                        ;2 13
       SBC    Obj2Y               ;3 16 Have we reached Object2's Y Coordinate?
       BPL    Waste11             ;2 18 ...If Not, Branch
       LDA    (Obj2Lo),Y          ;5 23 Get the Next Player01 Definition byte
       CMP    #GFX_DELIMITER      ;2 25 Check for delimiter
       BEQ    Waste4              ;2 27 If end of bitmap then Definition finished
       STA    GRP1                ;3 30 otherwise, display (early enough to display right away)
       INC    Obj2Lo              ;5 35 Goto next Player01 definition byte
PrintPlayer00: ;35 Print Player00 (Object01), Ball (Man) and Room.
       TXA                        ;2 37 Get Current Scan Line
       SEC                        ;2 39
       SBC    Obj1Y               ;3 42 Have we reached the Object1's Y coordinate?
       BPL    Waste11_2           ;2 44 If not then Branch
       LAX    (Obj1Lo),Y          ;5 49 Get the Next Player00 definition byte (give to X)
       CMP    #GFX_DELIMITER      ;2 51 Check for delimiter
       BEQ    Waste4_2            ;2 53 If end of bitmap then definition finished
       INC.w  Obj1Lo              ;6 59 Goto next Player00 definition byte...waste 1 cycle too
Check_For_Update: ;59
       DEC    PixelCount          ;5 64 decrement the line counter
       BNE    No_Playfield_Update ;2 66/67 if more lines remain, branch
       LDY    RoomDefIndex        ;3 69 Get room definition index
       INY                        ;2 71 bump index
       NOP                        ;2 73 (2 cycles free here)
       STX    GRP0                ;3 00 Display Player00 definition byte (if Wanted)
       LAX    (RoomLo),Y          ;5 05 Get first room definition byte (use LAX to preserve)
       STA    ENAM0               ;3 08 Enable left hand wall(if wanted-missle00)
Print_Top_Line: ;08
       STA    PF0                 ;3 11 ...and display
       INY                        ;2 13 bump index
       LDA    (RoomLo),Y          ;5 18 Get next room definition byte
       STA    PF1                 ;3 21 ...and display
       INY                        ;2 23 bump index
       LDA    (RoomLo),Y          ;5 28 Get next room definition byte
       STA    PF2                 ;3 31 ...and display
       INY                        ;2 33 bump index
       TXA                        ;2 35 Get PF0 definition back
       AND    #$0D                ;2 37 keep only low nybble of PF0
       ORA    #$20                ;2 39 merge in the ball size
       STA    CTRLPF              ;3 42 Store to playfield control
       LSR                        ;2 44
       LSR                        ;2 46
       STA    ENAM1               ;3 49 Enable right hand wall(if wanted-missle01)
       LDA    (RoomLo),Y          ;5 54 Get number of scanlines for this row
       STA    PixelCount          ;3 57 Store to line counter
       STY    RoomDefIndex        ;3 60 ...and save for Next Time
PrintPlayer01: ;60 Print Player01 (Object 02)
       LAX    ScanLineCnt         ;3 63 Get the scan line. Use LAX for subrtaction later
       DEX                        ;2 65 (save 3 cycles by using DEX instead of DEC)
       BEQ    KernelDone          ;2 67 Branch if display finished
       SBC    PlayerYadj          ;3 70 Have we reached the Man's Y Coordinate?
       AND    #$FC                ;2 72 Mask value to four either side (getting depth of 8)
       BEQ    Enable_Ball         ;2 74 If >4 on either end...
       BNE    Disable_Ball        ;2 01 always branch (skip ball display)

Waste11: ;19
       PLA                        ;4 23
       PHA                        ;3 26
       LDY    #$00                ;2 28 Clear Y (player 1 gfx)
Waste4: ;28
       STY.w  GRP1                ;4 32
       BEQ    PrintPlayer00       ;2 35 always branch

Waste11_2: ;45
       PLA                        ;4 49
       PHA                        ;3 52
       NOP                        ;2 54
Waste4_2: ;54
       LDX    #$00                ;2 56 Clear X (player 0 gfx)
       BEQ    Check_For_Update    ;2 59 always branch

GfxDrag0: ;21 bytes, roaming/swallowed dragon bitmap (States $00 and $02)
       .byte %00000110 ;     XX
       .byte %00001111 ;    XXXX
       .byte %11110011 ;XXXX  XX
       .byte %11111110 ;XXXXXXX
       .byte %00001110 ;    XXX
       .byte %00000100 ;     X
       .byte %00000100 ;     X
       .byte %00011110 ;   XXXX
       .byte %00111111 ;  XXXXXX
       .byte %01111111 ; XXXXXXX
       .byte %11100011 ;XXX   XX
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
       .byte %11000111 ;XX   XXX
       .byte %11111111 ;XXXXXXXX
       .byte %00111100 ;  XXXX
       .byte %00001000 ;    X
       .byte %10001111 ;X   XXXX
       .byte %11100001 ;XXX    X
       .byte %00111111 ;  XXXXXX
       .byte GFX_DELIMITER ;end

;       ORG  $10AB
;       .byte 0

       ORG  $1C92,0

GfxAuthor: ;96 bytes, Signature bitmap
       .byte %11110000 ;XXXX
       .byte %10000000 ;X
       .byte %10000000 ;X
       .byte %10000000 ;X
       .byte %11110100 ;XXXX X
       .byte %00000100 ;     X
       .byte %10000111 ;X    XXX
       .byte %11100101 ;XXX  X X
       .byte %10000111 ;X    XXX
       .byte %10000000 ;X
       .byte %00000101 ;     X X
       .byte %11100101 ;XXX  X X
       .byte %10100111 ;X X  XXX
       .byte %11100001 ;XXX    X
       .byte %10000111 ;X    XXX
       .byte %11100000 ;XXX
       .byte %00000000 ;
       .byte %11100000 ;XXX
       .byte %10100000 ;X X
       .byte %11110000 ;XXXX
       .byte %00000000 ;
       .byte %01000000 ; X
       .byte %11100000 ;XXX
       .byte %01000000 ; X
       .byte %01000000 ; X
       .byte %01000000 ; X
       .byte %00000000 ;
       .byte %11100000 ;XXX
       .byte %10100000 ;X X
       .byte %11100000 ;XXX
       .byte %10000000 ;X
       .byte %11100000 ;XXX
       .byte %00000000 ;
       .byte %00100000 ;  X
       .byte %00100000 ;  X
       .byte %11100000 ;XXX
       .byte %10100000 ;X X
       .byte %11100000 ;XXX
       .byte %00000000 ;
       .byte %00000000 ;
       .byte %00000000 ;
       .byte %00000000 ;
       .byte %10101000 ;X X X
       .byte %10101000 ;X X X
       .byte %10101000 ;X X X
       .byte %11111000 ;XXXXX
       .byte %00000000 ;
       .byte %11100000 ;XXX
       .byte %10100000 ;X X
       .byte %11110000 ;XXXX
       .byte %00000000 ;
       .byte %10000000 ;X
       .byte %11100000 ;XXX
       .byte %10001111 ;X   XXXX
       .byte %10001001 ;X   X  X
       .byte %00001111 ;    XXXX
       .byte %10001010 ;X   X X
       .byte %11101001 ;XXX X  X
       .byte %10000000 ;X
       .byte %10001110 ;X   XXX
       .byte %00001010 ;    X X
       .byte %11101110 ;XXX XXX
       .byte %10100000 ;X X
       .byte %11101000 ;XXX X
       .byte %10001000 ;X   X
       .byte %11101110 ;XXX XXX
       .byte %00001010 ;    X X
       .byte %10001110 ;X   XXX
       .byte %11100000 ;XXX
       .byte %10100100 ;X X  X
       .byte %10100100 ;X X  X
       .byte %00000100 ;     X
       .byte %00000000 ;
       .byte %00001000 ;    X
       .byte %00001110 ;    XXX
       .byte %00001010 ;    X X
       .byte %00001010 ;    X X
       .byte %00000000 ;
       .byte %00001110 ;    XXX
       .byte %00001010 ;    X X
       .byte %00001110 ;    XXX
       .byte %00001000 ;    X
       .byte %00001110 ;    XXX
       .byte %00000000 ;
       .byte %00000100 ;     X
       .byte %00001110 ;    XXX
       .byte %00000100 ;     X
       .byte %00000100 ;     X
       .byte %00000100 ;     X
       .byte %00000000 ;
       .byte %00000100 ;     X
       .byte %00001110 ;    XXX
       .byte %00000100 ;     X
       .byte %00000100 ;     X
       .byte %00000100 ;     X
       .byte GFX_DELIMITER ;end

GfxChallise: ;10 bytes, Challise bitmap
       .byte %10000001 ;X      X
       .byte %10000001 ;X      X
       .byte %11000011 ;XX    XX
       .byte %01111110 ; XXXXXX
       .byte %01111110 ; XXXXXX
       .byte %00111100 ;  XXXX
       .byte %00011000 ;   XX
       .byte %00011000 ;   XX
       .byte %01111110 ; XXXXXX
       .byte GFX_DELIMITER ;end

GfxKey: ;4 bytes, key bitmap
       .byte %00000111 ;     XXX
       .byte %11111101 ;XXXXXX X
       .byte %10100111 ;X X  XXX
       .byte GFX_DELIMITER ;end

       ORG  $1D00,0

GfxSurround: ;32 bytes, Surround bitmap
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte %11111111 ;XXXXXXXX
       .byte GFX_DELIMITER ;end

GfxPort01: ;17 bytes, Portcullis bitmap (state $1C...closed)
       .byte %11111110 ;XXXXXXX
       .byte %10101010 ;X X X X
GfxPort02: ;(state $18 and $20)
       .byte %11111110 ;XXXXXXX
       .byte %10101010 ;X X X X
GfxPort03: ;(state $14 and $24)
       .byte %11111110 ;XXXXXXX
       .byte %10101010 ;X X X X
GfxPort04: ;(state $10 and $28)
       .byte %11111110 ;XXXXXXX
       .byte %10101010 ;X X X X
GfxPort05: ;(state $0C and $2C)
       .byte %11111110 ;XXXXXXX
       .byte %10101010 ;X X X X
GfxPort06: ;(state $08 and $30)
       .byte %11111110 ;XXXXXXX
       .byte %10101010 ;X X X X
GfxPort07: ;(state $04 and $FF...open)
       .byte %11111110 ;XXXXXXX
       .byte %10101010 ;X X X X
GfxPort08:
       .byte %11111110 ;XXXXXXX
       .byte %10101010 ;X X X X
GfxPort09:
       .byte GFX_DELIMITER ;end

GfxBridge: ;25 bytes, bridge bitmap
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %01000010 ; X    X
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
       .byte GFX_DELIMITER ;end

GfxMagnet: ;9 bytes, Magnet bitmap
       .byte %00111100 ;  XXXX
       .byte %01111110 ; XXXXXX
       .byte %11100111 ;XXX  XXX
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
GfxMagnetEnd:
       .byte GFX_DELIMITER ;end (share next byte)

GfxSword: ;6 bytes, sword bitmap
       .byte %00100000 ;  X
       .byte %01000000 ; X
       .byte %11111111 ;XXXXXXXX
       .byte %01000000 ; X
       .byte %00100000 ;  X
       .byte GFX_DELIMITER ;end

GfxBat1: ;8 bytes, bat's 1st bitmap (state $03)
       .byte %10000001 ;X      X
       .byte %10000001 ;X      X
       .byte %11000011 ;XX    XX
       .byte %11000011 ;XX    XX
       .byte %11111111 ;XXXXXXXX
       .byte %01011010 ; X XX X
       .byte %01100110 ; XX  XX
       .byte GFX_DELIMITER ;end

GfxBat2: ;12 bytes, bat's 2nd bitmap (state $FF)
       .byte %00000000 ;
       .byte %00000000 ;
       .byte %00000000 ;
       .byte %00000000 ;
       .byte %00111100 ;  XXXX
       .byte %01011010 ; X XX X
       .byte %01100110 ; XX  XX
       .byte %11000011 ;XX    XX
       .byte %10000001 ;X      X
       .byte %10000001 ;X      X
       .byte %10000001 ;X      X
       .byte GFX_DELIMITER ;end

GfxDrag1: ;23 bytes, biting dragon bitmap (state $FF)
       .byte %10000000 ;X
       .byte %01000000 ; X
       .byte %00100110 ;  X  XX
       .byte %00011111 ;   XXXXX
       .byte %00001011 ;    X XX
       .byte %00001110 ;    XXX
       .byte %00011110 ;   XXXX
       .byte %00100100 ;  X  X
       .byte %01000100 ; X   X
       .byte %10001110 ;X   XXX
       .byte %00011110 ;   XXXX
       .byte %00111111 ;  XXXXXX
       .byte %01111111 ; XXXXXXX
       .byte %01111111 ; XXXXXXX
       .byte %01111111 ; XXXXXXX
       .byte %01111111 ; XXXXXXX
       .byte %00111110 ;  XXXXX
       .byte %00011100 ;   XXX
       .byte %00001000 ;    X
       .byte %11111000 ;XXXXX
       .byte %10000000 ;X
       .byte %11100000 ;XXX
       .byte GFX_DELIMITER ;end

GfxDrag2: ;18 bytes, dead dragon bitmap (state $02)
       .byte %00001100 ;    XX
       .byte %00001100 ;    XX
       .byte %00001100 ;    XX
       .byte %00001110 ;    XXX
       .byte %00011011 ;   XX XX
       .byte %01111111 ; XXXXXXX
       .byte %11001110 ;XX  XXX
       .byte %10000000 ;X
       .byte %11111100 ;XXXXXX
       .byte %11111110 ;XXXXXXX
       .byte %11111110 ;XXXXXXX
       .byte %01111110 ; XXXXXX
       .byte %01111000 ; XXXX
       .byte %00100000 ;  X
       .byte %01101110 ; XX XXX
       .byte %01000010 ; X    X
       .byte %01111110 ; XXXXXX
       .byte GFX_DELIMITER ;end

GfxDot: ;2 bytes, dot bitmap
       .byte %10000000 ;X
GfxNull:
       .byte GFX_DELIMITER ;end

GfxNum1: ;8 bytes, game selection #1 bitmap (state $01)
       .byte %00100000 ;  X
       .byte %01100000 ; XX
       .byte %00100000 ;  X
       .byte %00100000 ;  X
       .byte %00100000 ;  X
       .byte %00100000 ;  X
       .byte %01110000 ; XXX
       .byte GFX_DELIMITER ;end

GfxNum2: ;8 bytes, game selection #2 bitmap (state $03)
       .byte %01110000 ; XXX
       .byte %10001000 ;X   X
       .byte %00001000 ;    X
       .byte %00010000 ;   X
       .byte %00100000 ;  X
       .byte %01000000 ; X
       .byte %11111000 ;XXXXX
       .byte GFX_DELIMITER ;end

GfxNum3: ;8 bytes, game selection #3 bitmap (state $FF)
       .byte %01110000 ; XXX
       .byte %10001000 ;X   X
       .byte %00001000 ;    X
       .byte %00110000 ;  XX
       .byte %00001000 ;    X
       .byte %10001000 ;X   X
       .byte %01110000 ; XXX
       .byte GFX_DELIMITER ;end

;NOTE: Room definition data format is PF0,PF1,PF2, and # (of double scanlines)
;      Lower nybble of PF0 description:
;      Bit 0 (+$01) true if (R)eversed playfield
;      Bit 1 (+$02) true if left panel wanted
;      Bit 2 (+$04) true if playfield has priority over sprites (dark rooms, hide sprites)
;      Bit 3 (+$08) true if right panel wanted
;      The remainder of the bits are the value used for PF0.

Room00: ;Number Room Definition
Room12: ;Inside Yellow castle
       .byte $F1,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXRRRRRRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $31,$00,$00,$50      ;XX                                    RR - 80
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8

Room01: ;Below blue maze
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $03,$00,$00,$50      ;  |                                      - 80
       .byte $F1,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXRRRRRRRRRRRRRRRRRRRR - 8

Room04: ;Blue Maze, below black castle
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $01,$0C,$0C,$10      ;        XX    XX        RR    RR         - 16
       .byte $F1,$0C,$3C,$10      ;XXXX    XX    XXXX    RRRR    RR    RRRR - 16
       .byte $F1,$0C,$00,$10      ;XXXX    XX                    RR    RRRR - 16
       .byte $F1,$FF,$3F,$10      ;XXXXXXXXXXXXXXXXXX    RRRRRRRRRRRRRRRRRR - 16
       .byte $01,$30,$30,$10      ;      XX        XX    RR        RR       - 16
       .byte $F1,$33,$3F,$08      ;XXXX  XX  XXXXXXXX    RRRRRRRR  RR  RRRR - 8

Room05: ;Blue Maze, above entry
       .byte $F1,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXRRRRRRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $01,$00,$00,$10      ;                                         - 16
       .byte $F1,$FC,$FF,$10      ;XXXXXXXXXX  XXXXXXXXRRRRRRRR  RRRRRRRRRR - 16
       .byte $F1,$00,$C0,$10      ;XXXX              XXRR              RRRR - 16
       .byte $F1,$3F,$CF,$10      ;XXXX  XXXXXXXXXX  XXRR  RRRRRRRRRR  RRRR - 16
       .byte $01,$30,$CC,$10      ;      XX      XX  XXRR  RR      RR       - 16
       .byte $F1,$F3,$CC,$08      ;XXXXXXXX  XX  XX  XXRR  RR  RR  RRRRRRRR - 8

       ORG  $1E00,0

Room02: ;Below Yellow Castle
Room1E: ;Warren Robinett's room
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $01,$00,$00,$50      ;                                         - 80
       .byte $F1,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXRRRRRRRRRRRRRRRRRRRR - 8

Room03: ;Above the catacombs
       .byte $F1,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXRRRRRRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $09,$00,$00,$50      ;                                     |   - 80
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8

Room06: ;Bottom of Blue Maze
       .byte $F1,$F3,$0C,$08      ;XXXXXXXX  XX  XX        RR  RR  RRRRRRRR - 8 2lk scanlines
       .byte $01,$30,$0C,$10      ;      XX      XX        RR      RR       - 16
       .byte $F1,$3F,$0F,$10      ;XXXX  XXXXXXXXXX        RRRRRRRRRR  RRRR - 16
       .byte $F1,$00,$00,$10      ;XXXX                                RRRR - 16
       .byte $F1,$F0,$00,$10      ;XXXXXXXX                        RRRRRRRR - 16
       .byte $01,$30,$00,$10      ;      XX                        RR       - 16
       .byte $F1,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXRRRRRRRRRRRRRRRRRRRR - 8

Room07: ;Center of Blue Maze
       .byte $F1,$33,$3F,$08      ;XXXX  XX  XXXXXXXX    RRRRRRRR  RR  RRRR - 8 2lk scanlines
       .byte $01,$30,$3C,$10      ;      XX      XXXX    RRRR      RR       - 16
       .byte $F1,$FF,$3C,$10      ;XXXXXXXXXXXX  XXXX    RRRR  RRRRRRRRRRRR - 16
       .byte $01,$03,$3C,$10      ;          XX  XXXX    RRRR  RR           - 16
       .byte $F1,$33,$3C,$10      ;XXXX  XX  XX  XXXX    RRRR  RR  RR  RRRR - 16
       .byte $01,$33,$0C,$10      ;      XX  XX  XX        RR  RR  RR       - 16
       .byte $F1,$F3,$0C,$08      ;XXXXXXXX  XX  XX        RR  RR  RRRRRRRR - 8

Room08: ;Blue Maze Entry
       .byte $F1,$F3,$CC,$08      ;XXXXXXXX  XX  XX  XXRR  RR  RR  RRRRRRRR - 8 2lk scanlines
       .byte $01,$33,$0C,$10      ;      XX  XX  XX        RR  RR  RR       - 16
       .byte $F1,$33,$FC,$10      ;XXXX  XX  XX  XXXXXXRRRRRR  RR  RR  RRRR - 16
       .byte $01,$33,$00,$10      ;      XX  XX                RR  RR       - 16
       .byte $F1,$F3,$FF,$10      ;XXXXXXXX  XXXXXXXXXXRRRRRRRRRR  RRRRRRRR - 16
       .byte $01,$00,$00,$10      ;                                         - 16
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8

Room09: ;Catacombs Middle
       .byte $F5,$FF,$CC,$08      ;XXXXXXXXXXXX  XX  XXRR  RR  RRRRRRRRRRRR - 8 2lk scanlines
       .byte $05,$00,$CC,$10      ;              XX  XXRR  RR               - 16
       .byte $F5,$03,$CF,$10      ;XXXX      XXXXXX  XXRR  RRRRRR      RRRR - 16
       .byte $05,$03,$00,$10      ;          XX                RR           - 16
       .byte $F5,$F3,$FC,$10      ;XXXXXXXX  XX  XXXXXXRRRRRR  RR  RRRRRRRR - 16
       .byte $05,$33,$0C,$10      ;      XX  XX  XX        RR  RR  RR       - 16
       .byte $F5,$33,$CC,$08      ;XXXX  XX  XX  XX  XXRR  RR  RR  RR  RRRR - 8

Room0A: ;Catacombs Entry
       .byte $F5,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $05,$30,$00,$10      ;      XX                        RR       - 16
       .byte $F5,$30,$FF,$10      ;XXXX  XX    XXXXXXXXRRRRRRRRR   RR  RRRR - 16
       .byte $05,$30,$C0,$10      ;      XX          XXRR          RR       - 16
       .byte $F5,$F3,$C0,$10      ;XXXXXXXX  XX      XXRR      RR  RRRRRRRR - 16
       .byte $05,$03,$C0,$10      ;          XX      XXRR      RR           - 16
       .byte $F5,$FF,$CC,$08      ;XXXXXXXXXXXX  XX  XXRR  RR  RRRRRRRRRRRR - 8

Room0B: ;Catacombs Side
       .byte $F5,$33,$CC,$08      ;XXXX  XX  XX  XX  XXRR  RR  RR  RR  RRRR - 8 2lk scanlines
       .byte $05,$30,$CC,$10      ;      XX      XX  XXRR  RR      RR       - 16
       .byte $05,$3F,$CF,$10      ;      XXXXXX  XX  XXRR  RR  RRRRRR       - 16
       .byte $05,$00,$C0,$10      ;                  XXRR                   - 16
       .byte $05,$3F,$C3,$10      ;      XXXXXXXX    XXRR    RRRRRRRR       - 16
       .byte $05,$30,$C0,$10      ;      XX          XXRR          RR       - 16
       .byte $F5,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXRRRRRRRRRRRRRRRRRRRR - 8

Room0C: ;Right corridor, right of catacombs
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $09,$00,$00,$50      ;                                     |   - 80
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8

Room0D: ;Left corridor, below white castle
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $03,$00,$00,$50      ;  |                                      - 80
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8

Room0E: ;Below left corridor (2 screens below white castle)
Room1D: ;Below right corridor (below right panel room - #1, right of catacombs - #2/#3)
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $31,$00,$00,$50      ;XX                                    RR - 80
       .byte $F1,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXRRRRRRRRRRRRRRRRRRRR - 8

Room0F: ;White Castle (outside)
Room10: ;Black Castle (outside)
Room11: ;Yellow Castle (outside)
       .byte $F1,$FE,$15,$08      ;XXXXXXXXXXX X X X      R R R RRRRRRRRRRR - 8 2lk scanlines
       .byte $31,$03,$1F,$10      ;XX        XXXXXXX      RRRRRRR        RR - 16
       .byte $31,$03,$FF,$10      ;XX        XXXXXXXXXXRRRRRRRRRR        RR - 16
       .byte $31,$00,$FF,$10      ;XX          XXXXXXXXRRRRRRRR          RR - 16
       .byte $31,$00,$3F,$10      ;XX          XXXXXX    RRRRRR          RR - 16
       .byte $31,$00,$00,$10      ;XX                                    RR - 16
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8

       ORG  $1F00,0

Room13: ;Black Maze, right from entry
       .byte $F5,$F0,$FF,$08      ;XXXXXXXX    XXXXXXXXRRRRRRRR    RRRRRRRR - 8 2lk scanlines
       .byte $05,$00,$03,$10      ;            XX            RR             - 16
       .byte $F5,$FF,$03,$10      ;XXXXXXXXXXXXXX            RRRRRRRRRRRRRR - 16
       .byte $05,$00,$00,$10      ;                                         - 16
       .byte $35,$3F,$FF,$10      ;XX    XXXXXXXXXXXXXXRRRRRRRRRRRRRR    RR - 16
       .byte $05,$30,$00,$10      ;      XX                        RR       - 16
       .byte $F5,$F0,$FF,$08      ;XXXXXXXX    XXXXXXXXRRRRRRRR    RRRRRRRR - 8

Room14: ;Black Maze, Challise room
       .byte $F4,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXMMMMMMMMMMMMMMMMMMMM - 8 2lk scanlines
       .byte $04,$00,$C0,$10      ;                  XX                  MM - 16
       .byte $F4,$FF,$CF,$10      ;XXXXXXXXXXXXXXXX  XXMMMMMMMMMMMMMMMM  MM - 16
       .byte $04,$00,$0C,$10      ;                  XX                  MM - 16
       .byte $F4,$0F,$FF,$10      ;XXXX    XXXXXXXXXXXXMMMM    MMMMMMMMMMMM - 16
       .byte $04,$0F,$C0,$10      ;        XXXX      XX        MMMM      MM - 16
       .byte $34,$CF,$CC,$08      ;XX  XX  XXXX  XX  XXMM  MM  MMMM  MM  MM - 8

Room15: ;Black Maze, Dot's room
       .byte $F4,$F0,$FF,$08      ;XXXXXXXX    XXXXXXXXMMMMMMMM    MMMMMMMM - 8 2lk scanlines
       .byte $34,$00,$00,$10      ;XX                  MM                   - 16
       .byte $34,$3F,$FF,$10      ;XX    XXXXXXXXXXXXXXMM    MMMMMMMMMMMMMM - 16
       .byte $04,$30,$00,$10      ;      XX                  MM             - 16
       .byte $F4,$F0,$FF,$10      ;XXXXXXXX    XXXXXXXXMMMMMMMM    MMMMMMMM - 16
       .byte $34,$00,$03,$10      ;XX          XX     .MM          MM       - 16
       .byte $F4,$F0,$FF,$08      ;XXXXXXXX    XXXXXXXXMMMMMMMM    MMMMMMMM - 8

Room16: ;Black Maze Entry
       .byte $35,$CF,$CC,$08      ;XX  XX  XXXX  XX  XXRR  RR  RRRR  RR  RR - 8 2lk scanlines
       .byte $05,$C0,$CC,$10      ;    XX        XX  XXRR  RR        RR     - 16
       .byte $F5,$FF,$0F,$10      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 16
       .byte $05,$00,$00,$10      ;                                         - 16
       .byte $F5,$FF,$0F,$10      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 16
       .byte $05,$00,$00,$10      ;                                         - 16
       .byte $F5,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8

Room17: ;Red Maze, above black key's room
       .byte $F1,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXRRRRRRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $01,$00,$00,$10      ;                                         - 16
       .byte $F1,$FF,$0F,$10      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 16
       .byte $01,$00,$0C,$10      ;              XX        RR               - 16
       .byte $F1,$FF,$0C,$10      ;XXXXXXXXXXXX  XX        RR  RRRRRRRRRRRR - 16
       .byte $F1,$03,$CC,$10      ;XXXX      XX  XX  XXRR  RR  RR      RRRR - 16
       .byte $F1,$33,$CF,$08      ;XXXX  XX  XXXXXX  XXRR  RRRRRR  RR  RRRR - 8

Room18: ;Red Maze, above entry
       .byte $F1,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXRRRRRRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $01,$00,$C0,$10      ;                  XXRR                   - 16
       .byte $F1,$FF,$CF,$10      ;XXXXXXXXXXXXXXXX  XXRR  RRRRRRRRRRRRRRRR - 16
       .byte $01,$00,$CC,$10      ;              XX  XXRR  RR               - 16
       .byte $F1,$33,$FF,$10      ;XXXX  XX  XXXXXXXXXXRRRRRRRRRR  RR  RRRR - 16
       .byte $F1,$33,$00,$10      ;XXXX  XX  XX                RR  RR  RRRR - 16
       .byte $F1,$3F,$0C,$08      ;XXXX  XXXXXX  XX        RR  RRRRRR  RRRR - 8

Room19: ;Red Maze, black key's room
       .byte $F1,$33,$CF,$08      ;XXXX  XX  XXXXXX  XXRR  RRRRRR  RR  RRRR - 8 2lk scanlines
       .byte $F1,$30,$00,$10      ;XXXX  XX                        RR  RRRR - 16
       .byte $F1,$33,$FF,$10      ;XXXX  XX  XXXXXXXXXXRRRRRRRRRR  RR  RRRR - 16
       .byte $01,$33,$00,$10      ;      XX  XX                RR  RR  RRRR - 16
       .byte $F1,$FF,$00,$10      ;XXXXXXXXXXXX                RRRRRRRRRRRR - 16
       .byte $01,$00,$00,$10      ;                                         - 16
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8

Room1A: ;Red Maze Entry
       .byte $F1,$3F,$0C,$08      ;XXXX  XXXXXX  XX        RR  RRRRRR  RRRR - 8 2lk scanlines
       .byte $F1,$00,$0C,$10      ;XXXX          XX        RR          RRRR - 16
       .byte $F1,$FF,$0F,$10      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 16
       .byte $01,$30,$00,$10      ;      XX                        RR       - 16
       .byte $F1,$30,$00,$10      ;XXXX  XX                        RR  RRRR - 16
       .byte $01,$30,$00,$10      ;      XX                        RR       - 16
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8

Room1B: ;Just inside the black castle
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $31,$00,$00,$50      ;XX                                    RR - 80
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8

Room1C: ;Above right corridor (#1 - 2nd room in black castle, #2/#3 - right of catacombs)
       .byte $F1,$FF,$FF,$08      ;XXXXXXXXXXXXXXXXXXXXRRRRRRRRRRRRRRRRRRRR - 8 2lk scanlines
       .byte $31,$00,$00,$50      ;XX                                    RR - 80
       .byte $F1,$FF,$0F,$08      ;XXXXXXXXXXXXXXXX        RRRRRRRRRRRRRRRR - 8

       ORG  $1FF8
;do not use $fff8/$fff9...bankswitch hotspot locations
       .byte "2009"
       .word START1,START1






       ORG  $2000
       RORG $F000

START2:
       LDA    $FFF9               ;4 select this bank (filler)
       JMP    StartGame           ;3 go to cold start routine

Kernel: ;59
       LDA    $FFF8               ;4 63 select 1st bank
;72
       DEX                        ;2 74 (x=$FF)
       STX    VBLANK              ;3 01 Turn on VBLANK
       STY    GRP1                ;3 Clear any graphics for Player01
       STY    GRP0                ;3 Clear any graphics for Player00
       STY    ENABL               ;3 Clear any graphics for the ball (vertically-delayed)
       PLA                        ;4 Restore low pointer vectors...
       STA    Obj2Lo              ;3
       PLA                        ;4
       STA    Obj1Lo              ;3
       LDA    #$20                ;2
       BNE    Store_Timer         ;2 Always branch (save timer and return)

DoVSYNC: ;Perform VSYNC
       LDX    INTIM               ;4 Get Timer Output
       BNE    DoVSYNC             ;2 Wait for Time-Out
       LDA    #$02                ;2 (enable)
       STA    WSYNC               ;3 Wait for horizontal Blank
       STA    VBLANK              ;3 Start vertical Blanking
       STA    WSYNC               ;3 Wait for horizontal Blank
       STA    WSYNC               ;3 Wait for horizontal Blank
       STA    WSYNC               ;3 Wait for horizontal Blank
       STA    VSYNC               ;3 Start vertical sync
       STA    WSYNC               ;3 Wait for horizontal Blank
       STA    WSYNC               ;3 Wait for horizontal Blank
       STA    WSYNC               ;3 Wait for horizontal Blank
       STX    VSYNC               ;3 End vertical sync
       LDA    #$2C                ;2 Set clock interval to countdown next frame
Store_Timer:
       STA    TIM64T              ;4
       RTS                        ;6


PrintDisplay: ;draw a frame...
       LDA    PlayerX             ;3 Position Ball Strite to the X Coordinate (all frames)
       LDX    #$04                ;2
       JSR    PosSpriteX          ;6
       LDA    Obj1Lo              ;3 transfer low sprite0 pointer to stack
       PHA                        ;3
       LDA    Obj2Lo              ;3 transfer low sprite1 pointer to stack
       PHA                        ;3
PrintDisplay_1:
       LDX    INTIM               ;4 Wait for end of the current frame
       BNE    PrintDisplay_1      ;2
       STA    WSYNC               ;3 Wait for horizontal Blank
       STA    HMOVE               ;3 03 Apply Horizontal Motion
       STX    RoomDefIndex        ;3 06 Set room definition index
       STX    GRP1                ;3 09 Clear any graphics for sprite1
       STA    CXCLR               ;3 12 Clear Collision Latches
       INX                        ;2 14
       STX    VDELP1              ;3 17 vertically delay sprite1
       LDA    Control             ;3 20 Get 3-frame room number
       AND    #$7F                ;2 22 (strip inactive game bit)
       TAY                        ;2 24
       LDA    RoomDataTable0,Y    ;4 28 Get the room gfx LSB pointer
       STA    RoomLo              ;3 31
       LDA    RoomDataTable1,Y    ;4 35 Get the room gfx MSB pointer
       STA    RoomHi              ;3 38
       LDA    PlayerY             ;3 41 Get the Y Coordinate of the Man
       SEC                        ;2 43
       SBC    #$04                ;2 45 And Adjust it (By Four Scan Lines) for printing
       STA    PlayerYadj          ;3 48 (so Y Coordinate Specifies Middle)
       STA    HMCLR               ;3 51 Clear horizontal motion
       LDX    #$60                ;2 53
       STX    ScanLineCnt         ;3 56 Set initial Scan Line Count
       BPL    Kernel              ;2 59 always branch (watch out...max distance!)

PosSpriteX: ;Position Sprite X horizontally.
       LDY    #$02                ;2 Start with 10 clock cycles (to avoid HBLANK)
       SEC                        ;2
PosSpriteX_1:
       INY                        ;2 Divide the Coordinate wanted by fifteen
       SBC    #$0F                ;2 Get Course Horizontal Value...In Multiples of 5 Clock
       BCS    PosSpriteX_1        ;2 Cycles...(Therefore giving 15 Color Cycles)
       EOR    #$FF                ;2 Flip remainder to positive value (inverted)
       SBC    #$06                ;2 Convert to left or right of current position
       ASL                        ;2 Move to high nybble for TIA horizontal motion...
       ASL                        ;2
       ASL                        ;2
       ASL                        ;2
       STA    HMP0,X              ;4 Set horizontal (fine) motion of sprite
       STA    WSYNC               ;3 Wait for horizontal Blank
PosSpriteX_2:
       DEY                        ;2 Count down the color cycles...
       BPL    PosSpriteX_2        ;2 (these are 5 machine/15 color cycles)
       STA    RESP0,X             ;4 Reset the sprite, thus positioning it coursely
       RTS                        ;6


CacheObjects: ;Fill cache with two objects in this room.
       LDX    LastObj             ;3 Get Last Object
       LDA    #NullNumber         ;2 Set cache to no objects...
       STA    CurrentObject+2     ;3
       STA    CurrentObject+3     ;3
MoveNextObject:
       INX                        ;2 Goto the next object
       CPX    #NullNumber         ;2 Check if over maximum
       BCC    GetObjectsInfo      ;2 branch if OK
       LDX    #SurroundNumber     ;2 ...or reset to zero
GetObjectsInfo:
       LDA    Store1,X            ;4 Get low byte of object info
       STA    CurrentObject       ;3
       LDA    Store2,X            ;4 Get high byte of object info
       STA    CurrentObject+1     ;3
       LDY    #$00                ;2 reset pointer to this vector only
       LDA    (CurrentObject),Y   ;5 Get object's current room
       CMP    PlayerRoom          ;3 Is it in this room?
       BNE    CheckForMoreObjects ;2 If not lets try next object
       LDA    CurrentObject+2     ;3 Check first slot
       CMP    #NullNumber         ;2 If not a free slot...
       BNE    StoreObjectToPrint  ;2 ...then branch
       STX    CurrentObject+2     ;3 Store this object's number to print
CheckForMoreObjects:
       CPX    LastObj             ;3 Have we done all the objects?
       BNE    MoveNextObject      ;2 If not, continue
       BEQ    StoreCount          ;2 Always branch

StoreObjectToPrint:
       STX    CurrentObject+3     ;3 Store this object's number to print
StoreCount:
       STX    LastObj             ;3 If so, store current count for next time
SetupRoomPrint: ;Setup a room for print
       LDA    Control             ;3
       AND    #$80                ;2
       ORA    PlayerRoom          ;3 create a backup of the room number for 3 frames
       STA    Control             ;3
       LDY    PlayerRoom          ;3 Get current room number
       LDA    RoomDataTable2,Y    ;4 Get the room's foreground color
       JSR    ChangeColor         ;6 Change if necessary
       STA    COLUPF              ;3 Put in Playfield color register
       LDA    #$08                ;2 Get light grey background (all rooms)
       JSR    ChangeColor         ;6 Change if necessary
       STA    COLUBK              ;3 Put it in the Background color register
;Sort out their order...do 1st sprite...
       LDA    CurrentObject+2     ;3 If the object1 is the surround...
       BEQ    SwapPrintObjects    ;2 Then branch to swap (we want it as player01)
       CMP    #BridgeNumber       ;2 If the first object is the bridge...
       BNE    SetupObjectPrint    ;2 Then swap the objects (we want it as player01)
       LDA    CurrentObject+3     ;3 If the object2 is the surround...
       BEQ    SetupObjectPrint    ;2 branch (we want it as player01)
SwapPrintObjects:
       LDA    CurrentObject+2     ;3 Swap the objects to print
       LDX    CurrentObject+3     ;3
       STA    CurrentObject+3     ;3
       STX    CurrentObject+2     ;3
SetupObjectPrint:
       LDX    CurrentObject+2     ;3 Setup Object1 to print...get Object 1
       LDA    Store1,X            ;3 Get low pointer to it's dynamic information
       STA    CurrentObject       ;3 ...store
       LDA    Store2,X            ;3 Get high pointer to it's dynamic informtion
       STA    CurrentObject+1     ;3 ...store
       LDY    #$01                ;2 set initial Y index
       LDA    (CurrentObject),Y   ;5 Get Object1's X coordinate...
       STA    ObjX                ;3 and store for printing
       INY                        ;2 Bump index
       LDA    (CurrentObject),Y   ;5 Get Object1's Y coordinate...
       STA    Obj1Y               ;3 and store for printing
       LDY    Store3,X            ;4 Get low pointer to state value
       LDA    Store4,X            ;4 Get high pointer to state value
       BEQ    Animated_Object_1   ;2 If animated object then skip ahead
       STY    Obj1Lo              ;3 Otherwise, store for print...
       BNE    Static_Object_1     ;2 ...and skip ahead to store the MSB (always branch)
Animated_Object_1:
       JSR    GetObjectState      ;6 Find current state in the state information
       INY                        ;2 Index to the state's corresponding graphic pointer
       LDA    (CurrentObject),Y   ;5 Get Object1's low graphic address...
       STA    Obj1Lo              ;3 and store for printing
       INY                        ;2 bump index
       LDA    (CurrentObject),Y   ;5 Get Object1's high graphic address...
Static_Object_1:
       STA    Obj1Hi              ;3 and store for printing
       JSR    CheckFlipAndColor   ;6 Change if Necessary...
       STA    COLUP0              ;3 and set color
  IF FLIP_DRAGONS
       STY    REFP0               ;3 and store reflect
  ENDIF
       LDA    Store8,X            ;4 Get Object1's width...
       STA    NUSIZ0              ;3 Set sprite width
       LDX    #$00                ;2 Reposition here instead
       LDA    ObjX                ;3
       JSR    PosSpriteX          ;6
;do 2nd sprite...
       LDX    CurrentObject+3     ;3 Setup Object 2 to Print...get Object 2
       LDA    Store1,X            ;4 Get low pointer to it's dynamic information
       STA    CurrentObject       ;3
       LDA    Store2,X            ;4 Get high pointer to it's dynamic information
       STA    CurrentObject+1     ;3
       LDY    #$01                ;2 Set initial Y index
       LDA    (CurrentObject),Y   ;5 Get Object2's X coordinate...
       STA    ObjX                ;3 and store for printing
       INY                        ;2 Bump index
       LDA    (CurrentObject),Y   ;5 Get Object2's Y coordinate...
       STA    Obj2Y               ;3 and store for printing
       LDY    Store3,X            ;4 Get low pointer to state value
       LDA    Store4,X            ;4 Get high pointer to state value
       BEQ    Animated_Object_2   ;2 If animated object then skip ahead
       STY    Obj2Lo              ;3 Otherwise, store for printing...
       BNE    Static_Object_2     ;2 ...and skip ahead to store the MSB (always branch)
Animated_Object_2:
       JSR    GetObjectState      ;6 Find the current state in the state information
       INY                        ;2 Index to state's corresponding graphic pointer
       LDA    (CurrentObject),Y   ;5 Get Object2's low graphic address...
       STA    Obj2Lo              ;3 Store
       INY                        ;2 Bump index
       LDA    (CurrentObject),Y   ;5 Get Object2's high graphic address...
Static_Object_2:
       STA    Obj2Hi              ;3 store
       JSR    CheckFlipAndColor   ;6 Change if Necessary...
       STA    COLUP1              ;3 and set color
  IF FLIP_DRAGONS
       STY    REFP1               ;3 and store reflect
  ENDIF
       LDA    Store8,X            ;4 Get Object2's width...
       STA    NUSIZ1              ;3 (Used by bridge and invisible surround)
       LDX    #$01                ;2 Reposition here instead
       LDA    ObjX                ;3
       JMP    PosSpriteX          ;3 Jump into other subroutine (use it's RTS)



GetObjectState: ;Get pointer to current state
       LDA    Store5,X            ;4 Get low pointer to state information
       STA    CurrentObject       ;3
       LDA    Store6,X            ;4 Get high pointer to state information
       STA    CurrentObject+1     ;3
       LDA.wy VSYNC,Y             ;4 Get current state value
       LDY    #$FD                ;2 reset state table offset
GetObjectState_1:
       INY                        ;2 Goto next state in list of states (bump index by 3 bytes)
       INY                        ;2
       INY                        ;2
       CMP    (CurrentObject),Y   ;5 Have we found it in the list of states?
       BCC    GetObjectState_2    ;2 If near, then state found (exit subroutine)
       BNE    GetObjectState_1    ;2 Branch if state not found yet
GetObjectState_2:
       RTS                        ;6


CheckInput: ;Check for input
       INC    LoCnt               ;5 Increment low count (3x frames)
       BNE    GetJoystick         ;2  branch if no rollover (12.8 seconds)
       INC    HiCnt               ;5 Increment hight count if needed
       BNE    GetJoystick         ;2  branch if no rollover
       LDA    #$80                ;2  keep high bit set on rollover
       STA    HiCnt               ;3  ...store to attract mode
GetJoystick:
       LDA    SWCHA               ;4 Get joystick values
       CMP    #$FF                ;2 Check for any movement
       BNE    GetJoystick_2       ;2  ...branch if movement
       LDA    SWCHB               ;4 Get the console switches
       AND    #$03                ;2 Mask for the reset/select switches
       CMP    #$03                ;2 Have either of them been used?
       BEQ    GetJoystick_3       ;2 If not, branch
GetJoystick_2:
       LDA    #$00                ;2
       STA    HiCnt               ;3 Clear any attract mode
GetJoystick_3:
       RTS                        ;6



CheckFlipAndColor:
  IF FLIP_DRAGONS
       LDY    #$00                ;2 Set default reflect
       CMP    #>GfxDrag0          ;2 Check for chasing/biting dragons
       BNE    Dont_Flip_Sprite    ;2 ...and skip ahead if not
       LDA    CurrentStick        ;3
       AND    #$04                ;2
       BEQ    Dont_Flip_Sprite    ;2
       LDA    PlayerX             ;3 check against player's location
       SEC                        ;2
       SBC    ObjX                ;3
       BMI    Dont_Flip_Sprite    ;2
       LDY    #$08                ;2 Set reflect
Dont_Flip_Sprite:
  ENDIF
       LDA    Store7,X            ;4 Get Object's Color
ChangeColor: ;Change color of objects/screens...
       LSR                        ;2 Roll bit 0 to carry
       BCC    ChangeColor_2       ;2 If no carry, use original color value
       LDA    LoCnt               ;3 Otherwise...use frame counter for color
ChangeColor_2:
       BIT    HiCnt               ;3 Check for attract mode
       BPL    ChangeColor_3       ;2 ...branch if clear
       EOR    HiCnt               ;2 Otherwise, flip bits of attract with the original color
       AND    #$FB                ;2 ...and strip bit 2
ChangeColor_3:
       ASL                        ;2 Roll bits to their original position
       RTS                        ;6


StartGame: ;Cold Start
       SEI                        ;2
       CLD                        ;2
       LDY    #$00                ;2 Console = 2600
       TYA                        ;2
       LDX    $D0                 ;3
       CPX    #$2C                ;2
       BNE    SaveConsole         ;2
       LDX    $D1                 ;3
       CPX    #$A9                ;2
       BNE    SaveConsole         ;2
       INY                        ;2 Console = 7800
SaveConsole:
       TAX                        ;2
ResetAll:
       STA    VSYNC,X             ;4
       TXS                        ;2
       INX                        ;2
       BNE    ResetAll            ;2
       STY    Level               ;3 Save console type
       TYA                        ;2
       ASL                        ;2
       ASL                        ;2
       STA    CurrentStick        ;3 ...and to current status
       LDA    #$14                ;2 load a "dark" room number
       STA    SurroundR           ;3 ...and save it to the surround's room
  IF EDGE_PANELS
       LDA    #$06                ;2 Position missiles. Near left boundry...
       LDX    #$02                ;2 Set index to point at missile0
       JSR    PosSpriteX          ;6  ...and position it
       LDA    #$9D                ;2 Near right boundry...
  ELSE
       LDA    #$0D                ;2 Position missiles. Near left boundry...
       LDX    #$02                ;2 Set index to point at missile0
       JSR    PosSpriteX          ;6  ...and position it
       LDA    #$96                ;2 Near right boundry...
  ENDIF
       INX                        ;2 Bump index to point at missile1
       JSR    PosSpriteX          ;6  ...and position it
       STA    WSYNC               ;3 Wait for horizontal blank
       STA    HMOVE               ;3 Apply horizonal position
       BMI    SetupRoomObjects    ;2 Always branch- setup objects' initial rooms and positions

CheckGameStart: ;moved to main game loop to avoid triple-nesting
       LAX    SWCHB               ;4 Get the console switches
       EOR    #$FF                ;2 Flip (as reset active low)
       AND    CurrentStick        ;3 Compare with what was before
       LSR                        ;2 And check only the reset switch
       BCC    CheckReset          ;2 If no reset then branch
       TXA                        ;2
       LSR                        ;2 Check if switch still held
       BCS    KeepInactive        ;2 Branch if so
       LDA    ChalliseR           ;3 Get the room the Challise is in
       CMP    #WinRoom            ;2 Is it in the yellow castle?
       BEQ    SetupRoomObjects    ;2 Branch if game already won (and set up a new game)
       LDA    Control             ;3 Set the game to active
       AND    #$7F                ;2 Signal that the game has started
       STA    Control             ;3
KeepInactive:
       LDA    Control             ;3 Has the Game Started?
       BMI    SetupRoomObjects    ;2 If not then branch
       LDA    #$50                ;2 Get the X coordinate
       STA    PlayerX             ;3 Make it the current ball X coordinate
       STA    PrevX               ;3 Make it the previous ball X coordinate
       LDA    #$18                ;2 Get the Y coordinate
       STA    PlayerY             ;3 Make it the current ball Y coordinate
       STA    PrevY               ;3 Make it the previous ball Y coordinate
       LDA    #$11                ;2 Get the yellow castle room
       STA    PlayerRoom          ;3 Make it the current room
       STA    PrevRoom            ;3 Make it the previous room
       LDA    #$00                ;2
       STA    RDragonR+4          ;3 Set the red dragon's state to OK
       STA    YDragonR+4          ;3 Set the yellow dragon's state to OK
       STA    GDragonR+4          ;3 Set the green dragon's state to OK
       STA    NoteCnt             ;3
       BEQ    SignalGameStart     ;2 (Always branch)
CheckReset:
       LSR                        ;2 And check only the select switch
       BCC    StoreSwitches       ;2 Branch if select not being used
       LDA    PlayerRoom          ;3 Get the Current Room
       BNE    SetupRoomObjects    ;2 Branch if not the number room (game in progress)
       LDA    Level               ;3 Get current game level
       AND    #$0F                ;2 Strip off any sound
       CLC                        ;2
       ADC    #$02                ;2 Increment the level number (by two)
       CMP    #$06                ;2 Have we reached the maximum?
       BCC    ResetSetup          ;2  ...branch if not
       AND    #$01                ;2  Otherwise, set back to zero
ResetSetup:
       STA    Level               ;3 Store the new level number
SetupRoomObjects:
       LDA    Control             ;3 Set the game to inactive
       ORA    #$80                ;2
       STA    Control             ;3
SetupRoomObjects_1:
       LDA    #$00                ;2 Set the current room to the...
       STA    PlayerRoom          ;3 "Number" room
       STA    PrevRoom            ;3 And the previous room
       STA    PlayerY             ;3 And the previous Y coordinate
       STA    PrevY               ;3 (So can't be seen.)
       LDA    Level               ;3 Get the level number
       AND    #$0F                ;2 Strip off any sound
       STA    Level               ;3
       AND    #$0E                ;2 ...and the remaining bit before giving to the index
       TAY                        ;2
       LDA    Loc_4,Y             ;4 Get the low pointer to object locations
       STA    CurrentObject       ;3
       LDA    Loc_4 + 1,Y         ;4 Get the high pointer to object locations
       STA    CurrentObject+1     ;3
       LDY    #ByteNum            ;2 Copy all the objects dynamic information
SetupRoomObjects_2:
       LDA    (CurrentObject),Y   ;5 (the rooms and positions) into
       STA.wy DotR,Y              ;4            ...the working area
       DEY                        ;2
       BPL    SetupRoomObjects_2  ;2
       LDA    Level               ;3 Get the level number
       AND    #$0F                ;2 Strip off any sound
       CMP    #$04                ;2 Branch if level one
       BCC    SignalGameStart     ;2  ...Or two (Where all objects are in defined areas)
       JSR    DoVSYNC             ;6 Wait for VSYNC
       LDY    #RndNum             ;2 Randomize game objects...
  IF TEST_SEED
       LDA    #SEED               ;2 Check against known setup pattern
       STA    LoCnt               ;3
  ENDIF
RandomizeLevel3_2:
       LDA    LoCnt               ;3 Get the low input counter as seed
       LSR                        ;2
       LSR                        ;2
       LSR                        ;2 Generate a psudo-random room number
       LSR                        ;2
       LSR                        ;2
       SEC                        ;2
       ADC    LoCnt               ;3 Store the low input counter
       STA    LoCnt               ;3
       AND    #RndMax             ;2 Trim so represents a room value
       CMP    Loc_1 + 1,Y         ;4 If it is less than the lower bound for the object...
       BCC    RandomizeLevel3_2   ;2  ...then get another
       CMP    Loc_1 + 2,Y         ;4 If it equals or is less than the higher bound...
       BEQ    RandomizeLevel3_3   ;2 for the object...
       BCS    RandomizeLevel3_2   ;2 Then continue (branch if higher)
RandomizeLevel3_3:
       LDX    Loc_1,Y             ;4 Get the dynamic data index for this object
       STA    VSYNC,X             ;4 Store the new room value
       DEY                        ;2
       DEY                        ;2 Goto the next object
       DEY                        ;2
       BPL    RandomizeLevel3_2   ;2 ...Until all done
       JSR    PrintDisplay        ;6 Display rooms and objects
SignalGameStart:
       LDA    #NullNumber         ;2 Set no object carried
       STA    CarriedObj          ;3
StoreSwitches:
       LDA    SWCHB               ;4 Load current switches
       AND    #$0B                ;2 Keep B&W, Select, & Reset
       TAX                        ;4 Load current switches
       EOR    CurrentStick        ;3 Compare against last
       AND    #$08                ;2 Keep only B&W
       BEQ    No_Toggle           ;2 Branch if no change
       LDA    Level               ;3
       LSR                        ;2 Check for console type
       BCC    A2600               ;2 Branch if 2600
       TXA                        ;2 Console is 7800...get switches
       AND    #$08                ;2 Keep only B&W
       BNE    No_Toggle           ;2 Skip return position
A2600:
       LDA    CurrentStick        ;3
       EOR    #$04                ;2 Flip status
       .byte $2C                  ;4 Skip 2 bytes
No_Toggle:
       LDA    CurrentStick        ;3
       AND    #$F4                ;2
       STX    CurrentStick        ;3
       ORA    CurrentStick        ;3
       STA    CurrentStick        ;3
MainGameLoop:
       JSR    MakeSound           ;6 Make noise if necessary
       JSR    CheckInput          ;6 Check for controller input
       LDA    Control             ;3 Is The Game Active?
       BMI    NonActiveLoop       ;2 If Not, Branch
       LDA    ChalliseR           ;3 Get the room the Challise is in
       CMP    #WinRoom            ;2 Is it in the yellow castle?
       BNE    MainGameLoop_2      ;2 If Not, Branch
       LDA    Control             ;3
       ORA    #$80                ;2
       STA    Control             ;3 Set the game to inactive
       LDA    #$FF                ;2
       STA    NoteCnt             ;3 Set the note count to maximum
       LDA    Sound               ;3
       AND    #$0F                ;2 Set the noise type to end-noise
       STA    Sound               ;3  ...and store
MainGameLoop_2:
       LDY    #$00                ;2 Allow joystick read - all movement
       JSR    BallMovement        ;6 Check ball collisions and move ball
       JSR    MoveCarriedObject   ;6 Move any carried object
       JSR    DoVSYNC             ;6 Wait for VSYNC
       JSR    CacheObjects        ;6 Get next two objects to display
       JSR    PrintDisplay        ;6 Display the room and objects
       JSR    PickupPutdown       ;6 Deal with object pickup and putdown
       LDY    #$01                ;2 Disallow joystick read - move vertically only
       JSR    BallMovement        ;6 Check ball collisions and move ball
       JSR    Surround            ;6 Deal With Invisible Surround Moving
       JSR    DoVSYNC             ;6 Wait for VSYNC
       JSR    MoveBat             ;6 Move and deal with bat
       LDY    #Gates              ;2 For Each Portcullis
       JSR    Portals_2           ;6 Move and deal with portcullises
       JSR    PrintDisplay        ;6 Display the room and objects
       JSR    MoveGreenDragon     ;6 Move and deal with the green dragon
       JSR    MoveYellowDragon    ;6 Move and deal with the yellow dragon
       JSR    DoVSYNC             ;6 Wait for VSYNC
       LDY    #$02                ;2 Disallow stick read/bridge check - move horizontally only
       JSR    BallMovement        ;6 Check ball collisions and move ball
       JSR    MoveRedDragon       ;6 Move and deal with red dragon
       JSR    Magnet_Routine      ;6 Deal with the magnet
       JSR    PrintDisplay        ;6 Display the room and objects
       JMP    CheckGameStart      ;3 Check for Game Start

NonActiveLoop: ;Non-Active Game Loop
       JSR    DoVSYNC             ;6 Wait for VSYNC
       LDA    PlayerRoom          ;3
       BNE    Dont_Slow_Display   ;2 branch if level display is not shown
       JSR    PrintDisplay        ;6 Do another frame to slow down
       JSR    DoVSYNC             ;6
Dont_Slow_Display:
       JSR    PrintDisplay        ;6 Display the room and objects
       JSR    CacheObjects        ;6 Get next two objects to display
       JMP    CheckGameStart      ;3 Check for Game Start



FindObjHit: ;Find which object has hit object wanted
       BIT    CXPPMM              ;3 Get Player00-Player01
       BPL    FindObjHit_2        ;2 If nothing, Branch
       CPX    CurrentObject+2     ;3 Is object 1 the one being hit?
       BEQ    FindObjHit_3        ;2 If so, Branch
       CPX    CurrentObject+3     ;3 Is object 2 the one being hit?
       BEQ    FindObjHit_4        ;2 If so, Branch
FindObjHit_2:
       LDA    #NullNumber         ;2 Neither, select the null object
       RTS                        ;6
FindObjHit_3:
       LDA    CurrentObject+3     ;3 Yes, select the other
       RTS                        ;6
FindObjHit_4:
       LDA    CurrentObject+2     ;3 Yes, select the other
NoObject: ;No collision
       RTS                        ;6


PickupPutdown: ;Deal with object pickup and putdown
       LDA    INPT4               ;3 Get joystick trigger
       BMI    PickupPutdown_2     ;2 Branch if no change
       LDA    #NullNumber         ;2
       CMP    CarriedObj          ;3 If nothing is being carried...
       BEQ    PickupPutdown_2     ;2  ...then branch
       STA    CarriedObj          ;3 Otherwise, drop object
       LDA    Sound               ;2
       AND    #$0F                ;2 Strip any current sound
       ORA    #$50                ;2 Set noise type to five
       STA    Sound               ;3
       LDA    #$04                ;2
       STA    NoteCnt             ;3 set noise count to four
PickupPutdown_2: ;Check for collision
       BIT    CXP0FB              ;3 Get Ball-Player00 collision
       BVC    PickupPutdown_3     ;2 If nothing occured then branch
       LDX    CurrentObject+2     ;3 With Player00...get type of Player00
       JMP    CollisionDetected   ;3 Deal with collision

PickupPutdown_3:
       BIT    CXP1FB              ;3 Get Ball-Player01 collision
       BVC    PickupPutdown_4     ;2 If nothing has happened, branch
       LDX    CurrentObject+3     ;3 Get type of Player01...
CollisionDetected: ;Collision occurred (with something)...
       CPX    #SwordNumber        ;2 Is it carriable?
       BCC    NoObject            ;2 If not, branch
       CPX    CarriedObj          ;3 Is it the object being carried?
       BEQ    PickupObject        ;2 If so, branch (and actually pick it up)
       LDA    Sound               ;3
       AND    #$0F                ;2 Strip any current sound
       ORA    #$40                ;2 Set noise type to four
       STA    Sound               ;3
       LDA    #$04                ;2 Set noise type to four
       STA    NoteCnt             ;3
PickupObject:
       STX    CarriedObj          ;3 Set the object as being carried
       LDA    Store1,X            ;4 Get and store the low address
       TAX                        ;2
       LDA    CurrentStick        ;3 direction moved into the object...
       LSR                        ;2
       LSR                        ;2
       LSR                        ;2
       LSR                        ;2
       TAY                        ;2
       LDA    VBLANK,X            ;4 Get the object's X coordinate
       SEC                        ;2
       SBC    PlayerX             ;3 Subtract the ball's X coordinate
       CLC                        ;2
       ADC    X6pixels,Y          ;4
       STA    CarriedX            ;3  ...and store the difference
       LDA    WSYNC,X             ;4 Get the object's Y coordinate
       SEC                        ;2
       SBC    PlayerY             ;3 Subtract the Ball's Y coordinate
       CLC                        ;2
       ADC    Y6pixels,Y          ;4
       STA    CarriedY            ;3  ...and store the difference
PickupPutdown_4:
       RTS                        ;6



BallMovement: ;Check ball collisions and move ball using BIT to check the two upper bits
       LDA    CXBLPF              ;3 Get ball-playfield collision
       BMI    PlayerCollision     ;2 Branch if collision (Player-Wall)
       BIT    CXM0FB              ;3 Get ball-missile00 collision
       BVS    PlayerCollision     ;2 Branch if collision. (Player-Left Thin)
       BIT    CXM1FB              ;3 Get ball-missile01 collision
       BVC    BallMovement_2      ;2 Branch if no collision
       LDA    CurrentObject+3     ;3 If object2 (to print) is...
       CMP    #DotNumber          ;2  ...not the black dot
       BNE    PlayerCollision     ;2  ...then collide
BallMovement_2: ;using BIT to check the flags instead of AND...
       BIT    CXP0FB              ;3 Get sprite0-ball collision
       BVC    BallMovement_3      ;2 If no collision then branch
       LDA    CurrentObject+2     ;3 If object1 (to print) is not surround ($00)
       BNE    PlayerCollision     ;2  ...then branch (collision)
BallMovement_3:
       BIT    CXP1FB              ;3 Get sprite1-ball collision
       BVC    NoCollision         ;2 If no collision then branch
       LDA    CurrentObject+3     ;3 If player01 to print is the surround ($00)
       BEQ    NoCollision         ;2  ...then branch (no collision)
PlayerCollision: ;Player collided (with something)
       CPY    #$02                ;2 Are we checking for the bridge?
       BNE    ReadStick           ;2 If not, branch
       LDA    CarriedObj          ;3 Get the object being carried
       CMP    #BridgeNumber       ;2 Branch if it is the bridge
       BEQ    ReadStick           ;2
       LDA    PlayerRoom          ;3 Get the current room
       CMP    BridgeR             ;3 Is the bridge in this room?
       BNE    ReadStick           ;2 If not, branch
       LDA    PlayerX             ;3 Check going thru bridge...get the ball's X coordinate
       SEC                        ;2
       SBC    BridgeR+1           ;3 Subtract the bridge's X coordinate
       CMP    #$0A                ;2 If less than 10 pixels then forget it
       BCC    ReadStick           ;2
       CMP    #$17                ;2 If more than 23 pixels then forget it
       BCS    ReadStick           ;2
       LDA    BridgeR+2           ;3 Get the bridge's Y coordinate
       SEC                        ;2
       SBC    PlayerY             ;3 Subtract the ball's Y coordinate
       CMP    #$FC                ;2
       BCS    NoCollision         ;2 If more than $FC then going through bridge
       CMP    #$19                ;2 If more than $19 then forget it
       BCS    ReadStick           ;2
NoCollision: ;No collision (and going through bridge)
       LDA    CurrentStick        ;3
       ORA    #$F0                ;2 Reset the joystick input
       STA    CurrentStick        ;3
       LDA    PlayerRoom          ;3 Get the current room...
       STA    PrevRoom            ;3  ...and store temporarily
       LDA    PlayerX             ;3 Get the ball's X coordinate...
       STA    PrevX               ;3  ...and store temporarily
       LDA    PlayerY             ;3 Get the ball's Y coordinate...
       STA    PrevY               ;3  ...and Store Temporarily
ReadStick: ;Read Sticks
       TYA                        ;2 Check for zero (all movement)
       BNE    ReadStick_2         ;2 If not, don't bother with joystick read
       LDA    CurrentStick        ;3
       AND    #$0F                ;2
       STA    CurrentStick        ;3
       LDA    SWCHA               ;4 Read joysticks
       AND    #$F0                ;2
       ORA    CurrentStick        ;3
       STA    CurrentStick        ;3
ReadStick_2:
       LDA    PrevRoom            ;3 Get Temporary room...
       STA    PlayerRoom          ;3  ...and make it the current room
       LDA    PrevX               ;3 Get temporary X coordinate...
       STA    PlayerX             ;3  ...and make it the man's X coordinate
       LDA    PrevY               ;3 Get temporary Y coordinate...
       STA    PlayerY             ;3  ...and make it the man's Y coordinate
       LDA    CurrentStick        ;3 Get the Joystick position
       ORA    ReadStick_3,Y       ;4 Merge out movement not allowed in this phase
       STA    ObjDir              ;3 And store cooked movement
       LDX    #PlayerRoom         ;2 Point to ball's co-ordinates
       LDY    #$03                ;2 Set the delta for the ball (3 pixels)
       BNE    MoveGroundObject    ;2 Branch to move the ball


PBCollision: ;Get player-ball collision
       CMP    CurrentObject+2     ;3 Is it the first object?
       BEQ    PBCollision_2       ;2 YES - Then Branch
       CMP    CurrentObject+3     ;3 Is it the second object?
       BEQ    PBCollision_3       ;2 YES - Then Branch
       LDA    #$00                ;2 Otherwise, nothing
       RTS                        ;6
PBCollision_2:
       LDA    CXP0FB              ;3 Get player00-ball collision
       AND    #$40                ;2
       RTS                        ;6
PBCollision_3:
       LDA    CXP1FB              ;3 Get player01-ball collision
       AND    #$40                ;2
MoveCarriedObject_2:
       RTS                        ;6


MoveCarriedObject: ;Move the carried object
       LDX    CarriedObj          ;3 Get the object being carried
       CPX    #NullNumber         ;2 Check if nothing
       BEQ    MoveCarriedObject_2 ;2  if so, return
       LDA    Store1,X            ;4 Get and store the low address
       TAX                        ;2 transfer to the X register
       LDA    PlayerRoom          ;3 Get the current room
       STA    VSYNC,X             ;4 ...and store the object's current room
       LDA    PlayerX             ;3 Get the ball's X coordinate
       CLC                        ;2
       ADC    CarriedX            ;3 Add the X difference
       STA    VBLANK,X            ;4 ...and store as the object's X coordinate
       LDA    PlayerY             ;3 Get the ball's Y coordinate
       CLC                        ;2
       ADC    CarriedY            ;3 Add the Y difference
       JMP    MoveObjectDelta     ;3 ...And jump to store


MoveGameObject:
       LDX    Delta1              ;3 Get dynamic data address
       LDA    ObjDir              ;3 Get Movement
       BNE    MoveGameObject_2    ;2 If movement then branch
       LDA    RSYNC,X             ;4 Use old movement
MoveGameObject_2:
       STA    RSYNC,X             ;4 Store the new movement
       LDY    Delta2              ;3 Get the object's Delta
MoveGroundObject: ;Move the object
       AND    #$F0                ;2 Check for motion
       BEQ    MoveObjectNoDelta   ;2 skip if no movement
       CMP    #$F0                ;2
       BEQ    MoveObjectNoDelta   ;2 skip if no movement
       STY    ObjAddress          ;3 store delta
       CPY    #$00                ;2
       BEQ    MoveObjectNoDelta   ;2 skip if no movement
       LSR                        ;2
       LSR                        ;2
       LSR                        ;2
       LSR                        ;2
       TAY                        ;2
       LDA    Xeor,Y              ;4 flip direction if applicable
       CMP    #$80                ;2
       BEQ    SkipXadjustment     ;2
       EOR    ObjAddress          ;3 Load the delta
       CLC                        ;2
       BPL    NotLeft             ;2
       ADC    #$01                ;2
NotLeft:
       ADC    VBLANK,X            ;4 Add the X coordinate
       STA    VBLANK,X            ;4 Store the X coordinate
SkipXadjustment:
       LDA    Yeor,Y              ;4 flip direction if applicable
       CMP    #$80                ;2
       BEQ    MoveObjectNoDelta   ;2
       EOR    ObjAddress          ;3 Load the delta
       CLC                        ;2
       BPL    NotDown             ;2
       ADC    #$01                ;2
NotDown:
       ADC    WSYNC,X             ;4 Add the Y coordinate
MoveObjectDelta:
       STA    WSYNC,X             ;4 Store the Y coordinate (60)
MoveObjectNoDelta:
       LDY    #Gates              ;2 Set to do the three portcullises
MoveGroundObject_2:
       STY    ObjAddress          ;3
       LDA.wy YGateR,Y            ;4 Get the portal state
       CMP    #$1C                ;2 Is it in a closed state?
       BEQ    GetPortal           ;2 If not, next portal
       LDY    ObjAddress          ;3 Deal with object moving out of a castle...get port number
       LDA    VSYNC,X             ;4 Get object's room number
       CMP    EntryRoomOffsets,Y  ;4 Is it in a castle entry room
       BNE    GetPortal           ;2 If not, next portal
       LDA    WSYNC,X             ;4 Get the object's Y coordinate
       CMP    #$05                ;2 Is it above $0D i.e at the bottom?
       BPL    GetPortal           ;2 If so then branch
       LDA    CastleRoomOffsets,Y ;4 Get the castle room
       STA    VSYNC,X             ;4 And put the object in the castle room
       LDA    #$50                ;2
       STA    VBLANK,X            ;4 Set the object's new X coordinate
       LDA    #$24                ;2
       STA    WSYNC,X             ;4 Set the new object's Y coordinate
       LDA    #$00                ;2
       STA.wy YGateR,Y            ;4 Set the portcullis state to Open
       RTS                        ;6
GetPortal:
       LDY    ObjAddress          ;3 Get the portcullis number
       DEY                        ;2  ...goto next...
       BPL    MoveGroundObject_2  ;2  ...and continue for all of them
       LDY    VSYNC,X             ;4 Check and Deal with Up...get the object's room
       LDA    WSYNC,X             ;4 Get the object's Y coordinate
       CMP    #$61                ;2 Has it reached above the top
       BMI    DealWithLeft        ;2 If not, branch
       LDA    #$05                ;2 Set new Y coordinate to bottom
       STA    WSYNC,X             ;4
       LDA    RoomDataTable4,Y    ;4 Get the room number to the North
       JMP    GetNewRoom          ;3
DealWithLeft: ;Check and Deal with Left
       LDA    VBLANK,X            ;4 Get the object's X coordinate
       CMP    #$03                ;2 Is it Three or less?
       BCC    DealWithLeft_2      ;2 If so, branch (off to left)
       CMP    #$F0                ;2 Is it's $F0 or more
       BCC    DealWithDown        ;2 If not, branch (off to left)
DealWithLeft_2:
       CPX    #PlayerRoom         ;2 Are we dealing with the ball?
       BEQ    DealWithLeft_3      ;2 If so then branch
       LDA    #$9A                ;2 Set new X coordinate for the others
       BNE    DealWithLeft_4      ;2 Always branch
DealWithLeft_3:
       LDA    #$9E                ;2 Set new X coordinate for the ball
DealWithLeft_4:
       STA    VBLANK,X            ;4 Store the next X coordinate
       LDA    RoomDataTable7,Y    ;4 Get the room number to the Left
       JMP    GetNewRoom          ;3
DealWithDown: ;Check and Deal with Down
       LDA    WSYNC,X             ;4 Get object's Y coordinate
       CMP    #$05                ;2 If it's greater than $0D
       BCS    DealWithRight       ;2  ...then branch
       LDA    #$60                ;2 Set new Y coordinate
       STA    WSYNC,X             ;4
       LDA    RoomDataTable6,Y    ;4 Get the room number to the South
       JMP    GetNewRoom          ;3
DealWithRight: ;Check and Deal with Right
       LDA    VBLANK,X            ;4 Get the object's X coordinate
       CPX    #PlayerRoom         ;2 Are we dealing with the ball?
       BNE    DealWithRight_2     ;2 Branch if not
       CMP    #$9F                ;2 Has the object reached the right?
       BCC    MovementReturn      ;2 Branch if not
       LDA    VSYNC,X             ;4 Get the Ball's Room
       CMP    #$03                ;2 Is it room #3 (Right to secret room)
       BNE    DealWithRight_3     ;2 Branch if not
       LDA    DotR                ;3 Check the room of the black dot
       CMP    #$15                ;2 Is it in the hidden room area?
       BEQ    DealWithRight_3     ;2 If so, Branch
       LDA    #$1E                ;2 Manually change to secret room
       STA    VSYNC,X             ;4 And make it current
       LDA    #$03                ;2 Set the X coordinate
       STA    VBLANK,X            ;4
       RTS                        ;6
DealWithRight_2:
       CMP    #$9B                ;2 Has the object reached the right of screen?
       BCC    MovementReturn      ;2 Branch if not (no room change)
DealWithRight_3:
       LDA    #$03                ;2 Set the next X coordinate
       STA    VBLANK,X            ;4
       LDA    RoomDataTable5,Y    ;4 Get the room number to the Right
GetNewRoom: ;Adjust room for different levels
       BPL    AdjustRoomLevel_2   ;2 Branch if high bit not set
       SEC                        ;2
       SBC    #$80                ;2 Remove the high bit...
       STA    Temp                ;3 ...and store the difference # line
       LDA    Level               ;3 Get the level number
       AND    #$0E                ;2 Strip off any sound
       LSR                        ;2 Divide it by two
       CLC                        ;2
       ADC    Temp                ;3 Add to the difference # line
       TAY                        ;2
       LDA    RoomDiffs,Y         ;4 Use as an offset to get the next room
AdjustRoomLevel_2:
       STA    VSYNC,X             ;4 ...and store as new object's room
MovementReturn:
       RTS                        ;6



GetLikedObject: ;Find liked object and get movement
       LDY    #$FF                ;2 Set index to zero
GetLikedObject_2:
       INY                        ;2
       STY    Index               ;3 Set the index number
       LDA    (ObjLst),Y          ;5 Get first object
       BEQ    GetLikedObject_3    ;2 check for table end right away
       STA    Temp                ;3
       TAX                        ;2
       LDA    VSYNC,X             ;4 Get object1's room
       STA    ObjDir              ;3 temp (xroom)
       INY                        ;2
       LDA    (ObjLst),Y          ;5 Get second object
       TAX                        ;2
       LDA    VSYNC,X             ;4 Compare with object2's room
       CMP    ObjDir              ;3 temp (xroom)
       BNE    GetLikedObject_2    ;2 If not the same room then branch
       CPX    ObjCr               ;3 Have we matched the second object
       BEQ    GetLikedObject_2    ;2  ...for difficulty? (if so, carry on)
       LDA    Temp                ;3 temp (x)
       CMP    ObjCr               ;3 Have we matched the first object?
       BEQ    GetLikedObject_2    ;2 Branch if so (56 cycles each pass)
GetLikedObject_4: ;Work out object's movement
       LDY    Temp                ;3 temp (x)
       LDA    #$FF                ;2 Set object movement to none
       STA    ObjDir              ;3
       LDA    VSYNC,X             ;4 Get object2's room
       CMP.wy VSYNC,Y             ;4 Compare it with object's room
       BNE    GetLikedObject_8    ;2 If not the same, forget it
       LDA    VBLANK,X            ;4 Get Object2's X coordinate
       CMP.wy VBLANK,Y            ;4 Get Object1's X coordinate
       BCC    GetLikedObject_5    ;2 If Object2 to left of Object1 then Branch
       BEQ    GetLikedObject_6    ;2 If Object2 = Object1 then Branch
       LDA    #$7F                ;2 Signal a move right
       BNE    GetLikedObject_5b   ;2 Always branch
GetLikedObject_5:
       LDA    #$BF                ;2 Signal a move left
GetLikedObject_5b:
       CPY    #MagnetR            ;2 bugfix...
       BNE    GetLikedObject_5c   ;2
       EOR    #$C0                ;2
GetLikedObject_5c:
       STA    ObjDir              ;3
GetLikedObject_6:
       LDA    WSYNC,X             ;4 Get Object2's Y Coordinate
       CMP.wy WSYNC,Y             ;4 Get Object1's X Coordinate
       BCC    GetLikedObject_7    ;2 If Object2 Below Object1 Then Branch
       BEQ    GetLikedObject_8    ;2 If Object2 on Object1 Then Branch
       LDA    #$EF                ;2 Signal a move up
       BNE    GetLikedObject_9    ;2 always branch
GetLikedObject_8:
       LDA    ObjDir              ;3 Get the Move
       RTS                        ;6
GetLikedObject_7:
       LDA    #$DF                ;2 Signal a move down
GetLikedObject_9:
       AND    ObjDir              ;3 Get object movement
       CPY    #MagnetR            ;2 bugfix...
       BNE    GetLikedObject_3    ;2
       LDY    MagSave             ;3 bugfix...Check backup
       CPY    #MAGNET_HEIGHT      ;2 ...for lowest valid position
       BCC    GetLikedObject_3    ;2 Branch if clear (and continue current course)
       EOR    #$30                ;2 ...otherwise, reverse direction
GetLikedObject_3: ;tag moved down here
       STA    ObjDir              ;3
       RTS                        ;6



MoveRedDragon: ;Move the Red Dragon
       LDA    #<RedDragMatrix     ;2
       LDY    #$03                ;2
       LDX    #RDragonNumber      ;2
       BNE    MoveDragon          ;2 always branch
MoveYellowDragon: ;Move the Yellow Dragon
       LDA    #<YelDragMatrix     ;2
       LDX    #YDragonNumber      ;2
       BNE    DragonSlowSpeed     ;2 always branch
MoveGreenDragon: ;Move the Green Dragon
       LDA    #<GreenDragonMatrix ;2
       LDX    #GDragonNumber      ;2
DragonSlowSpeed:
       LDY    #$02                ;2
MoveDragon:
       STA    ObjLst              ;3 Set Low Address of Object Store
       STY    Delta2              ;3 Set the Green Dragon's Delta
       LDA    #>RedDragMatrix     ;2 NOTE: all dragon matrixes must be on the same page!
       STA    ObjLst+1            ;3 Set High Address of Object Store
       STX    ObjectNumber        ;3 Save Object were dealing with
       LDA    Store1,X            ;4 Get the Object's Dynamic Data
       TAX                        ;2
       LDA    NUSIZ0,X            ;4 Get the Object's State
       BNE    MoveDragon_6        ;2 Branch if not roaming
       BIT    SWCHB               ;4 Dragon Normal (State 1)...Read console switches
       BMI    MoveDragon_2        ;2 If expert, Branch (don't ignore sword)
       LDA    #SwordR             ;2 Set Easy - Ignore Sword
MoveDragon_2:
       STA    ObjCr               ;3 Store Difficulty
       STX    Delta1              ;3 Store Dynamic Data Address
       JSR    GetLikedObject      ;6 Get liked object and movement
       JSR    MoveGameObject      ;6
       LDA    ObjectNumber        ;3 Get Object
       JSR    PBCollision         ;6  ...And get the Player-Ball collision
       BEQ    MoveDragon_4        ;2 If None Then Branch
       LDA    Level               ;3 Get Level Number
       AND    #$0E                ;2 Strip off any sound
       TAY                        ;2 Create Lookup
       BIT    SWCHB               ;4 Get Console Switches
       BVC    MoveDragon_3        ;2 If P0="B" Then Branch
       INY                        ;2 Bump Lookup
MoveDragon_3:
       LDA    DragonDiff,Y        ;4 Get New State
       STA    NUSIZ0,X            ;4 Store as Dragon's State (Open Mouthed)
       LDA    PrevX               ;3
       STA    VBLANK,X            ;4 Get Temp Ball X Coord and Store as Dragon's
       LDA    PrevY               ;3
       STA    WSYNC,X             ;4 Get Temp Ball Y Coord and Store as Dragon's
       LDA    Sound               ;3
       AND    #$0F                ;2 Strip any current sound
       ORA    #$10                ;2
       STA    Sound               ;3 Set Noise Type to 01 (roar)
       LDA    #$10                ;2
       STA    NoteCnt             ;3 Set Noise Count to 17 frames
MoveDragon_4:
       STX    ObjAddress          ;3 Store Object's Dynamic Data Address
       LDX    ObjectNumber        ;3 Get the Object Number
       JSR    FindObjHit          ;6 See if another object has hit the dragon
       LDX    ObjAddress          ;3 Get the Object Address
       CMP    #SwordNumber        ;2 Has the Sword hit the Dragon?
       BNE    MoveDragon_5        ;2 If Not, Branch
       LDA    #$01                ;2 Set the State to 01 (Dead)
       STA    NUSIZ0,X            ;4
       LDA    Sound               ;3
       AND    #$0F                ;2 Strip any current sound
       ORA    #$30                ;2 Set Sound Three
       STA    Sound               ;3
       LDA    #$10                ;2 Set a Noise count of 17 frames
       STA    NoteCnt             ;3
MoveDragon_5:
       RTS                        ;6
MoveDragon_6:
       CMP    #$01                ;2 Is it in State 01 (Dead)?
       BEQ    MoveDragon_9        ;2 Branch if So (Return)
       CMP    #$02                ;2 Is it int State 02 (Normal #2)?
       BNE    MoveDragon_7        ;2 Branch if Not
       LDA    VBLANK,X            ;4 Get the Dragon's X Coordinate
       CLC                        ;2 Normal Dragon State 2 (Eaten Ball)
       ADC    #$03                ;2 Adjust
       STA    PlayerX             ;3  ...and store as the ball's X coordinate
       STA    PrevX               ;3  ...and previous X coordinate
       LDA    WSYNC,X             ;4 Get the Dragon's Y coordinate
       SEC                        ;2
       SBC    #$0A                ;2 Adjust
       STA    PlayerY             ;3  ...and store as the ball's Y coordinate
       STA    PrevY               ;3  ...and the previous Y coordinate
       LDY    VSYNC,X             ;4 Get the Dragon's Current Room
       STY    PlayerRoom          ;3 Store as the Ball's Current Room
       STY    PrevRoom            ;3  ...and Previous Room
       RTS                        ;6
MoveDragon_7: ;Dragon Roaring
       TAY                        ;2 transfer state to Y
       INY                        ;2 bump state
       STY    NUSIZ0,X            ;4 Save it's State
       CPY    #$FC                ;2 Is it near the end?
       BCC    MoveDragon_9        ;2 If Not, Branch
       LDA    ObjectNumber        ;3 Get the Dragon's Number
       JSR    PBCollision         ;6 Check if the Ball is colliding
       BEQ    MoveDragon_9        ;2 If not, Branch
       LDA    #$02                ;2 Set the State to State 02 : Eaten
       STA    NUSIZ0,X            ;4
       LDA    Sound               ;3
       AND    #$0F                ;2 Strip any current sound
       ORA    #$20                ;2 Set noise two
       STA    Sound               ;3
       LDA    #$10                ;2 Set the Count of Noise to 17 frames
       STA    NoteCnt             ;3
       LDA    #$9B                ;2 Get the Maximum X Coordinate
       CMP    VBLANK,X            ;4 Compare with the Dragon's X Coordinate
       BEQ    MoveDragon_8        ;2
       BCS    MoveDragon_8        ;2
       STA    VBLANK,X            ;4 If too large then Use It
MoveDragon_8:
       LDA    #$0F                ;2 Set Minimum Y Coordinate
       CMP    WSYNC,X             ;4 Compare with the Dragon's Y Coordinate
       BCC    MoveDragon_9        ;2
       STA    WSYNC,X             ;4 If Too Small, set as Dragon's Y coordinate
MoveDragon_9:
       RTS                        ;6



MoveBat: ;Move and deal with the bat
       LDX    BatR+4              ;3 Get the Bat State
       INX                        ;2 bump state
       TXA                        ;2
       AND    #$07                ;2 Has it Reached the Maximum?
       STA    BatR+4              ;3 (reset to zero when bit 3 is reached)
       LDA    BatR+6              ;3 Get the Bat Fed-Up Value
       BEQ    MoveBat_3           ;2 If Bat Fed-Up then Branch
       INC    BatR+6              ;5 Increment its value for next time
       LDA    BatR+3              ;3 Get the Bat's Movement
       LDX    #BatR               ;2 Position to Bat
       LDY    #$03                ;2 Get the Bat's Deltas
       JSR    MoveGroundObject    ;6 Move the Bat
       JMP    MoveBat_4           ;3 Update the Bat's Object
MoveBat_3: ;Bat Fed-Up
       LDA    #BatR               ;2 Store the Bat's Dynamic Data Address
       STA    Delta1              ;3
       LDA    #$03                ;2 Set the Bat's Delta
       STA    Delta2              ;3
       LDA    #<BatMatrix         ;2 Set the Low Address of Object Store
       STA    ObjLst              ;3
       LDA    #>BatMatrix         ;2 Set the High Address of Object Store
       STA    ObjLst+1            ;3
       LDA    BatR+5              ;3 Get Object being Carried by Bat...
       STA    ObjCr               ;3  ...and Copy
       JSR    GetLikedObject      ;6 Get liked object and movement
       JSR    MoveGameObject      ;6 Move the Bat
       LDY    Index               ;3 Get Object Liked Index
       LDA    (ObjLst),Y          ;5 Look up the Object Found in the Table
       BEQ    MoveBat_4           ;2 If nothing found then Forget it
       INY                        ;2
       LAX    (ObjLst),Y          ;5 Get the Object Wanted
       LDA    VBLANK,X            ;4 Get the Object's X Coordinate
       SEC                        ;2 ;See if Bat Can pick up Object
       SBC    BatR+1              ;3 Find the difference with the Bat's X coordinate
       CLC                        ;2
       ADC    #$04                ;2 Adjust so Bat in middle of object
       AND    #$F8                ;2 Is Bat within Seven Pixels?
       BNE    MoveBat_4           ;2 If not, no pickup possible
       LDA    WSYNC,X             ;4 Get the Object's Y Coordinate
       SEC                        ;2
       SBC    BatR+2              ;3 Find the Difference with the Bat's Y Coordinate
       CLC                        ;2
       ADC    #$04                ;2 Adjust
       AND    #$F8                ;2 Is the Bat within Seven Pixels?
       BNE    MoveBat_4           ;2 If not, No Pickup Possible
       STX    BatR+5              ;3 Store Object as Being Carried
       LDA    #$10                ;2 Reset the Bat Fed Up Time
       STA    BatR+6              ;3
MoveBat_4: ;Move Object Being Carried by Bat
       LDX    BatR+5              ;3 Get Object Being Carried by Bat
       LDA    BatR                ;3 Get the Bat's Room
       STA    VSYNC,X             ;4 Store this as the Object's Room
       LDA    BatR+1              ;3 Get the Bat's X coordinate
       CLC                        ;2
       ADC    #$08                ;2 Adjust to the Right
       STA    VBLANK,X            ;4 Make it the Object's X coordinate
       LDA    BatR+2              ;3 Get the Bat's Y Coordinate
       STA    WSYNC,X             ;4 Store it as the Object's Y Coordinate
       LDA    BatR+5              ;3 Get the Object Being Carried by the Bat
       LDY    CarriedObj          ;3 Get the Object Being Carried by the Ball
       CMP    Store1,Y            ;4 Are the the Same?
       BNE    MoveBat_5           ;2 If not, Branch
       LDA    #NullNumber         ;2 Set Nothing Carried
       STA    CarriedObj          ;3
MoveBat_5:
       RTS                        ;6



Portals_2: ;Deal with Portcullis and Collisions
       LDX    PortOffsets,Y       ;4 Get the Portcullises offset number
       JSR    FindObjHit          ;6 See if an Object Collided with it
       STA    Temp                ;3  ...Store that Object
       LDX    YGateR,Y            ;4
       CMP    KeyOffsets,Y        ;4 Is it the Associated Key?
       BNE    Portals_3           ;2 If not then Branch
       INX                        ;2 Change it's state to open/close it
Portals_3:
       STX    YGateR,Y            ;4 Save its State
       CPX    #$1C                ;2 Is it Closed?
       BEQ    Portals_9           ;2 Yes - then Branch
       LDA    PortOffsets,Y       ;4 Get Portcullis number
       JSR    PBCollision         ;6 Get the Player-Ball Collision
       BEQ    Portals_4           ;2 If Not Then Branch
       STY    ObjAddress          ;3 Save Y
       LDA    EntryRoomOffsets,Y  ;4 Look up Castle entry room for this port
       STA    PlayerRoom          ;3  ...and make it the current room
       LDY    ObjAddress          ;3 Retrieve Y
       LDX    #$00                ;2 Set the Portcullis to Open state
       STX    YGateR,Y            ;4
       LDX    #PlayerRoom         ;2 Set to the Castle
       BNE    Portals_6_2         ;2 Branch & put the Ball in the Castle
Portals_4:
       LDX    Temp                ;3 Get Object
       CPX    #NullNumber         ;2 Is it nothing?
       BEQ    Portals_7           ;2 If so, Branch
       STY    ObjAddress          ;3 Save Y
       LDA    Store1,X            ;4 Get the object's ram address
       TAX                        ;2 transfer the object's ram address to X
Portals_6:
       LDA    EntryRoomOffsets,Y  ;4 Look up Castle entry room for this port
       STA    VSYNC,X             ;4 Make it the object's Room
Portals_6_2:
       LDA    #$08                ;2 Give the Object a new Y coordinate
       STA    WSYNC,X             ;4
Portals_7:
       LDX    YGateR,Y            ;4 Get its State
       BEQ    Portals_9           ;2 If open, then Branch
       INX                        ;2
       CPX    #$38                ;2 Has it reached the maximum state
       BNE    Portals_8           ;2 If not, Branch
       LDX    #$00                ;2 Set to Open State
Portals_8:
       STX    YGateR,Y            ;4
Portals_9:
       DEY                        ;2 Goto the next portcullis
       BPL    Portals_2           ;2 Do next portcullis
       RTS                        ;6



Magnet_Routine: ;Deal with magnet
       LDA    MagnetR+2           ;3 Get Magnet's Y Coordinate
       STA    MagSave             ;3 Make a backup
       SEC                        ;2
       SBC    #MAGNET_HEIGHT      ;2 Adjust to it's "Poles"
       STA    MagnetR+2           ;3
       LDA    #$00                ;2 Con Difficulty!
       STA    ObjCr               ;3
       LDA    #<MagnetMatrix      ;2 Set Low Address of Object Store
       STA    ObjLst              ;3
       LDA    #>MagnetMatrix      ;2 Set High Address of Object Store
       STA    ObjLst+1            ;3
       JSR    GetLikedObject      ;6 Get Liked Object and Set Movement
       LDA    ObjDir              ;3 Get Movement
       BEQ    Mag_2               ;2 If None, then Forget It
       LDY    #$01                ;2 Set Delta to One
       JSR    MoveGroundObject    ;6 Move Object
Mag_2:
       LDA    MagSave             ;3 Restore magnet's actual position
       STA    MagnetR+2           ;3
       RTS                        ;6



Surround: ;Deal with Invisible Surround Moving
       LDY    PlayerRoom          ;3 Get the Current Room
       LDA    RoomDataTable1,Y    ;4 Get the Room's CTRL
       AND    #$20                ;2 Is it Invisible?
       BEQ    Surround_4          ;2 If not, branch
       STY    SurroundR           ;3 Store room as the Invisible Surround's room
       LDA    PlayerX             ;3 Get the Ball's X Coordinate
       SEC                        ;2
       SBC    #$10                ;2 Adjust for Surround
       BPL    Surround_3          ;2    00 to 7F OK
       CMP    #$F0                ;2 Is it close to the right edge?
       BCC    Surround_2          ;2 Branch if not
       LDA    #$01                ;2 Flick surround to the left
       BNE    Surround_3          ;2 Always branch
Surround_2:
       LDA    #$7F                ;2 If higher than $7F, keep boundry on the right
Surround_3:
       STA    SurroundR+1         ;3 ...and store as surround's X coordinate
       LDA    PlayerY             ;3 Get the Ball's Y Coordinate
       CLC                        ;2
       ADC    #$0E                ;2 Adjust for Surround...
Surround_4:
       STA    SurroundR+2         ;3  ...and store as surround's Y coordinate
       RTS                        ;6



MakeSound: ;Make a noise
       LDA    Sound               ;3 Get the Noise Type
       LSR                        ;2
       LSR                        ;2
       LSR                        ;2
       LSR                        ;2
       TAX                        ;2
       LDY    NoteCnt             ;3 Check Note Count
       BEQ    NoiseSaveVolume     ;2 Branch if no noise to be made
MakeSound_2:
       DEY                        ;2
       STY    NoteCnt             ;3 Goto the Next Note
       TYA                        ;2
       LDY    #$00                ;2
       DEX                        ;2 Game Over?
       BMI    NoiseGameOver       ;2
       DEX                        ;2 Roar?
       BMI    NoiseRoar           ;2
       DEX                        ;2 Man Eaten?
       BMI    EatenNoise          ;2
       DEX                        ;2 Dying Dragon?
       BMI    DragDieNoise        ;2
       DEX                        ;2 Picking up Object?
       BMI    NoiseGetObject      ;2
       DEX                        ;2 Dropping Object?
       BPL    NoiseSaveVolume     ;2 branch if not
;Sounds...Set A (Frequency), Y (Volume), and X (distortion)...& branch to NoiseSave
;Noise 5 : Dropping Object
       EOR    #$03                ;2 Reverse it as pitch goes up
NoiseGetObject: ;Noise 4 : Picking up an Object
       LDY    #$05                ;2
       BNE    NoiseSaveX          ;2 always branch
NoiseGameOver: ;Noise 0 : winning game
       STA    COLUPF              ;3 Color-Luminance Playfield
       TAX                        ;2 Audio-Control 0
       LSR                        ;2
       TAY                        ;2 Audio-Volume 0
       LSR                        ;2
       LSR                        ;2
       BPL    NoiseSave           ;2 always branch
NoiseRoar: ;Noise 1 : Roar
       TAY                        ;2 Put the Note Count In the Volume
       LSR                        ;2
       LDX    #$03                ;2 If it was even...
       BCS    SetVolume           ;2  ...then branch
       LDX    #$08                ;2 Get a different audio control value
SetVolume:
       TYA                        ;2 Set the Volume
       CLC                        ;2
       ADC    #$70                ;2 Set the Frequency
NoiseFrequency:
       LSR                        ;2 Divide by Four
       LSR                        ;2
       BPL    NoiseSave           ;2 always branch
DragDieNoise: ;Noise 3 : Dying Dragon
       TAY                        ;2 Put the Note Count In the Volume
       EOR    #$1F                ;2
       LDX    #$04                ;2 Set the Audio Control
       BNE    NoiseSave           ;2 always branch
EatenNoise: ;Noise 2 : Man Eaten
       LSR                        ;2 Divide note count by 2
       ADC    #$08                ;2
       TAY                        ;2
       LDA    NoteCnt             ;3
       EOR    #$0F                ;2
NoiseSaveX:
       LDX    #$06                ;2
NoiseSave:
       STA    AUDF0               ;3 Store in Frequency for Channel 0
       STX    AUDC0               ;3 Set a Noise on Channel 0
NoiseSaveVolume:
       STY    AUDV0               ;3 Set Volume on Channel 0
       RTS                        ;6

;       ORG  $283D
;       RORG $F83D
;       .byte 0


       ORG  $2D0D,0
       RORG $FD0D,0

MagnetMatrix: ;13 bytes, Magnet Object Matrix
       .byte MagnetR,YKeyR          ;Yellow Key, Magnet
       .byte MagnetR,WKeyR          ;White Key, Magnet
       .byte MagnetR,BKeyR          ;Black Key, Magnet
       .byte MagnetR,SwordR         ;Sword, Magnet
       .byte MagnetR,BridgeR        ;Bridge, Magnet
       .byte MagnetR,ChalliseR      ;Challise, Magnet
ReadStick_3: ;Joystick Merge Values
       .byte $00,$C0,$30 ;No change, No horizontal, No vertical...first byte shared w/previous

AuthorInfo: ;Object $4 : Author's signature data
       .byte $1E,$50,$61          ;Room 1E, (50, 69)

Xeor: ;used for all object movement
       .byte $80,$80,$80,$80,$80,$00,$00,$00,$80,$FF,$FF,$FF ;next 4 shared
Yeor: ;used for all object movement
       .byte $80,$80,$80,$80,$80,$FF,$00,$80,$80,$FF,$00,$80,$80,$FF,$00,$80

BatMatrix: ;21 bytes, Bat Object Matrix
       .byte BatR,ChalliseR         ;Bat,Challise
       .byte BatR,SwordR            ;Bat,Sword
       .byte BatR,BridgeR           ;Bat,Bridge
       .byte BatR,YKeyR             ;Bat,Yellow Key
       .byte BatR,WKeyR             ;Bat,White Key
       .byte BatR,BKeyR             ;Bat,Black Key
       .byte BatR,RDragonR          ;Bat,Red Dragon
       .byte BatR,YDragonR          ;Bat,Yellow Dragon
       .byte BatR,GDragonR          ;Bat,Green Dragon
       .byte BatR,MagnetR           ;Bat,Magnet
       .byte $00                    ;matrix end

BatStates: ;6 bytes, Bat animation list
       .byte $03,<GfxBat1,>GfxBat1     ;State 03 at &FD1A
       .byte $FF,<GfxBat2,>GfxBat2     ;State FF as &FD22

DragonDiff: ;Dragon Difficulty
       .byte $D0,$E8                ;Level 1 : Am, Pro
       .byte $F0,$F6                ;Level 2 : Am, Pro
       .byte $F0,$F6                ;Level 3 : Am, Pro

CastleRoomOffsets: ;Castle Rooms (Yellow, White, Black)...the screen of the castle itself
       .byte $11,$0F,$10

KeyOffsets: ;Keys #1, #2, #3  (Yellow, White, Black)
     .byte YKeyNumber,WKeyNumber,BKeyNumber

PortOffsets: ;Portcullis #1, #2, #3
     .byte YGateNumber,WGateNumber,BGateNumber

PortInfo1: ;Yellow castle's portcullis
       .byte $11,$4D,$29            ;Room number, X, Y
PortInfo2: ;White castle's portcullis
       .byte $0F,$4D,$29            ;Room number, X, Y
PortInfo3: ;Black castle's portcullis
       .byte $10,$4D,$29            ;Room number, X, Y

PortStates: ;39 bytes, Portcullis animation list
       .byte $04,<GfxPort07,>GfxPort07 ;State 04 -Open
       .byte $08,<GfxPort06,>GfxPort06 ;State 08
       .byte $0C,<GfxPort05,>GfxPort05 ;State 0C
       .byte $10,<GfxPort04,>GfxPort04 ;State 10
       .byte $14,<GfxPort03,>GfxPort03 ;State 14
       .byte $18,<GfxPort02,>GfxPort02 ;State 18
       .byte $1C,<GfxPort01,>GfxPort01 ;State 1C -Closed
       .byte $20,<GfxPort02,>GfxPort02 ;State 20
       .byte $24,<GfxPort03,>GfxPort03 ;State 24
       .byte $28,<GfxPort04,>GfxPort04 ;State 28
       .byte $2C,<GfxPort05,>GfxPort05 ;State 2C
       .byte $30,<GfxPort06,>GfxPort06 ;State 30
       .byte $FF,<GfxPort07,>GfxPort07 ;State FF -Open

NumberStates: ;9 bytes, ;Object #5 States
       .byte $01,<GfxNum1,>GfxNum1  ;State 1 as FCF4
       .byte $03,<GfxNum2,>GfxNum2  ;State 3 as FD04
       .byte $FF,<GfxNum3,>GfxNum3  ;State FF as FD0C

;NOTE: Game1Objects and Game2Objects must hold the same number of bytes
;98 bytes total for both tables
Game1Objects: ;Object locations (room and coordinate) for game 01
       .byte $15,$51,$0A            ;Black dot (Room, X, Y)
       .byte $0E,$50,$18,$00,$00    ;Red Dragon (Room, X, Y, Movement, State)
       .byte $01,$50,$18,$00,$00    ;Yellow Dragon (Room, X, Y, Movement, State)
       .byte $1D,$50,$18,$00,$00    ;Green Dragon (Room, X, Y, Movement, State)
       .byte $1B,$80,$18            ;Magnet (Room,X,Y)
       .byte $12,$20,$18            ;Sword (Room,X,Y)
       .byte $1C,$30,$18            ;Challise (Room,X,Y)
       .byte $04,$29,$2F            ;Bridge (Room,X,Y)
       .byte $11,$20,$38            ;Yellow Key (Room,X,Y)
       .byte $0E,$20,$38            ;White Key (Room,X,Y)
       .byte $1D,$20,$38            ;Black Key (Room,X,Y)
       .byte $1C                    ;Portcullis State
       .byte $1C                    ;Portcullis State
       .byte $1C                    ;Portcullis State
       .byte $1A,$20,$18,$00,$00    ;Bat (Room, X, Y, Movement, State)
       .byte $78,$00                ;Bat (Carrying, Fed-Up)
Game2Objects: ;Object locations (room and coordinate) for Games 02 and 03.
       .byte $15,$51,$0A            ;Black Dot (Room,X,Y)
       .byte $14,$50,$18,$A0,$00    ;Red Dragon (Room,X,Y,Movement,State)
       .byte $19,$50,$18,$A0,$00    ;Yellow Dragon (Room,X,Y,Movement,State)
       .byte $04,$50,$18,$A0,$00    ;Green Dragon (Room,X,Y,Movement,State)
       .byte $0E,$80,$18            ;Magnet (Room,X,Y)
       .byte $11,$20,$18            ;Sword (Room,X,Y)
       .byte $14,$30,$18            ;Challise (Room,X,Y)
       .byte $0B,$40,$38            ;Bridge (Room,X,Y)
       .byte $09,$20,$38            ;Yellow Key (Room,X,Y)
       .byte $06,$20,$38            ;White Key (Room,X,Y)
       .byte $19,$20,$38            ;Black Key (Room,X,Y)
       .byte $1C                    ;Portcullis State
       .byte $1C                    ;Portcullis State
       .byte $1C                    ;Portcullis State
       .byte $02,$20,$18,$90,$00    ;Bat (Room,X,Y,Movement,State)
       .byte $78,$00                ;Bat (Carrying, Fed-Up)

       ORG  $2E00
       RORG $FE00

Objects:
;NOTE: SurroundData must be listed first, NullData last.
Store1: ;LSB of object room/x/y table (or ram location of room #'s)
SurroundData:
       .byte <SurroundR    ;$00 Invisible Surround's room (LSB)
YGateData:
       .byte <PortInfo1    ;$01 Yellow castle gate's room (LSB)
WGateData:
       .byte <PortInfo2    ;$02 White castle gate's room (LSB)
BGateData:
       .byte <PortInfo3    ;$03 Black castle gate's room (LSB)
AuthorData:
       .byte <AuthorInfo   ;$04 Author signature's room (LSB)
NumberData:
       .byte <NumberInfo   ;$05 Game selection digit's room (LSB)
RDragonData:
       .byte <RDragonR     ;$06 Red Dragon's room (LSB)
YDragonData:
       .byte <YDragonR     ;$07 Yellow Dragon's room (LSB)
GDragonData:
       .byte <GDragonR     ;$08 Green Dragon's room (LSB)
SwordData:
       .byte <SwordR       ;$09 Sword's room (LSB)
BridgeData:
       .byte <BridgeR      ;$0A Bridge's room (LSB)
YKeyData:
       .byte <YKeyR        ;$0B Yellow key's room (LSB)
WKeyData:
       .byte <WKeyR        ;$0C White key's room (LSB)
BKeyData:
       .byte <BKeyR        ;$0D Black key's room (LSB)
BatData:
       .byte <BatR         ;$0E Bat's room (LSB)
DotData:
       .byte <DotR         ;$0F Secret dot's room (LSB)
ChalliseData:
       .byte <ChalliseR    ;$10 Challise's room (LSB)
MagnetData:
       .byte <MagnetR      ;$11 Magnet's room (LSB)
NullData:
       .byte <NullR        ;$12 Null object's room (LSB)

Store2: ;MSB of object room/x/y table (zero if ram)
       .byte $00           ;$00 Invisible Surround's room (MSB)
       .byte >PortInfo1    ;$01 Yellow castle gate's room (MSB)
       .byte >PortInfo2    ;$02 White castle gate's room (MSB)
       .byte >PortInfo3    ;$03 Black castle gate's room (MSB)
       .byte >AuthorInfo   ;$04 Author signature's room (MSB)
       .byte >NumberInfo   ;$05 Game selection digit's room (MSB)
       .byte $00           ;$06 Red Dragon's room (MSB)
       .byte $00           ;$07 Yellow Dragon's room (MSB)
       .byte $00           ;$08 Green Dragon's room (MSB)
       .byte $00           ;$09 Sword's room (MSB)
       .byte $00           ;$0A Bridge's room (MSB)
       .byte $00           ;$0B Yellow key's room (MSB)
       .byte $00           ;$0C White key's room (MSB)
       .byte $00           ;$0D Black key's room (MSB)
       .byte $00           ;$0E Bat's room (MSB)
       .byte $00           ;$0F Secret dot's room (MSB)
       .byte $00           ;$10 Challise's room (MSB)
       .byte $00           ;$11 Magnet's room (MSB)
       .byte $00           ;$12 Null object's room (MSB)

Store3: ;use ram address of state for animated sprites, gfx address for static objects...
       .byte <GfxSurround  ;$00 Invisible Surround's state (LSB)
       .byte <YGateR       ;$01 Yellow castle gate's state (LSB)
       .byte <WGateR       ;$02 White castle gate's state (LSB)
       .byte <BGateR       ;$03 Black castle gate's state (LSB)
       .byte <GfxAuthor    ;$04 Author signature's state (LSB)
       .byte <Level        ;$05 Game selection digit's state (LSB)
       .byte <RDragonR+4   ;$06 Red Dragon's state (LSB)
       .byte <YDragonR+4   ;$07 Yellow Dragon's state (LSB)
       .byte <GDragonR+4   ;$08 Green Dragon's state (LSB)
       .byte <GfxSword     ;$09 Sword's state (LSB)
       .byte <GfxBridge    ;$0A Bridge's state (LSB)
       .byte <GfxKey       ;$0B Yellow key's state (LSB)
       .byte <GfxKey       ;$0C White key's state (LSB)
       .byte <GfxKey       ;$0D Black key's state (LSB)
       .byte <BatR+4       ;$0E Bat's state (LSB)
       .byte <GfxDot       ;$0F Secret dot's state (LSB)
       .byte <GfxChallise  ;$10 Challise's state (LSB)
       .byte <GfxMagnet    ;$11 Magnet's state (LSB)
       .byte <GfxNull      ;$12 Null object's state (LSB)

Store4: ;use $00 for animated sprites, gfx address MSB for static objects...
       .byte >GfxSurround  ;$00 Invisible Surround's state (MSB)
       .byte $00           ;$01 Yellow castle gate's state (MSB)
       .byte $00           ;$02 White castle gate's state (MSB)
       .byte $00           ;$03 Black castle gate's state (MSB)
       .byte >GfxAuthor    ;$04 Author signature's state (MSB)
       .byte $00           ;$05 Game selection digit's state (MSB)
       .byte $00           ;$06 Red Dragon's state (MSB)
       .byte $00           ;$07 Yellow Dragon's state (MSB)
       .byte $00           ;$08 Green Dragon's state (MSB)
       .byte >GfxSword     ;$09 Sword's state (MSB)
       .byte >GfxBridge    ;$0A Bridge's state (MSB)
       .byte >GfxKey       ;$0B Yellow key's state (MSB)
       .byte >GfxKey       ;$0C White key's state (MSB)
       .byte >GfxKey       ;$0D Black key's state (MSB)
       .byte $00           ;$0E Bat's state (MSB)
       .byte >GfxDot       ;$0F Secret dot's state (MSB)
       .byte >GfxChallise  ;$10 Challise's state (MSB)
       .byte >GfxMagnet    ;$11 Magnet's state (MSB)
       .byte >GfxNull      ;$12 Null object's state (MSB)

Store5: ;only used for animated sprites...
       .byte $00           ;$00 Invisible Surround's list of states (LSB)
       .byte <PortStates   ;$01 Yellow castle gate's list of states (LSB)
       .byte <PortStates   ;$02 White castle gate's list of states (LSB)
       .byte <PortStates   ;$03 Black castle gate's list of states (LSB)
       .byte $00           ;$04 Author signature's list of states (LSB)
       .byte <NumberStates ;$05 Game selection digit's list of states (LSB)
       .byte <DragonStates ;$06 Red Dragon's list of states (LSB)
       .byte <DragonStates ;$07 Yellow Dragon's list of states (LSB)
       .byte <DragonStates ;$08 Green Dragon's list of states (LSB)
       .byte $00           ;$09 Sword's list of states (LSB)
       .byte $00           ;$0A Bridge's list of states (LSB)
       .byte $00           ;$0B Yellow key's list of states (LSB)
       .byte $00           ;$0C White key's list of states (LSB)
       .byte $00           ;$0D Black key's list of states (LSB)
       .byte <BatStates    ;$0E Bat's list of states (LSB)
       .byte $00           ;$0F Secret dot's list of states (LSB)
       .byte $00           ;$10 Challise's list of states (LSB)
       .byte $00           ;$11 Magnet's list of states (LSB)
       .byte $00           ;$12 Null object's list of states (LSB)

Store6: ;only used for animated sprites...
       .byte $00           ;$00 Invisible Surround's list of states (MSB)
       .byte >PortStates   ;$01 Yellow castle gate's list of states (MSB)
       .byte >PortStates   ;$02 White castle gate's list of states (MSB)
       .byte >PortStates   ;$03 Black castle gate's list of states (MSB)
       .byte $00           ;$04 Author signature's list of states (MSB)
       .byte >NumberStates ;$05 Game selection digit's list of states (MSB)
       .byte >DragonStates ;$06 Red Dragon's list of states (MSB)
       .byte >DragonStates ;$07 Yellow Dragon's list of states (MSB)
       .byte >DragonStates ;$08 Green Dragon's list of states (MSB)
       .byte $00           ;$09 Sword's list of states (MSB)
       .byte $00           ;$0A Bridge's list of states (MSB)
       .byte $00           ;$0B Yellow key's list of states (MSB)
       .byte $00           ;$0C White key's list of states (MSB)
       .byte $00           ;$0D Black key's list of states (MSB)
       .byte >BatStates    ;$0E Bat's list of states (MSB)
       .byte $00           ;$0F Secret dot's list of states (MSB)
       .byte $00           ;$10 Challise's list of states (MSB)
       .byte $00           ;$11 Magnet's list of states (MSB)
       .byte $00           ;$12 Null object's list of states (MSB)

Store7: ;Color (if odd, object is to Flash)
       .byte $28           ;$00 Invisible Surround's color (Orange)
       .byte $00           ;$01 Yellow castle gate's color (Black)
       .byte $00           ;$02 White castle gate's color (Black)
       .byte $00           ;$03 Black castle gate's color (Black)
       .byte $CB           ;$04 Author signature's color (Flash)
       .byte $C8           ;$05 Game selection digit's color (Green)
       .byte $36           ;$06 Red Dragon's color (Red)
       .byte $1A           ;$07 Yellow Dragon's color (Yellow)
       .byte $C8           ;$08 Green Dragon's color (Green)
       .byte $1A           ;$09 Sword's color (Yellow)
       .byte $66           ;$0A Bridge's color (Purple)
       .byte $1A           ;$0B Yellow key's color (Yellow)
       .byte $0E           ;$0C White key's color (White)
       .byte $00           ;$0D Black key's color (Black)
       .byte $00           ;$0E Bat's color (Black)
       .byte $08           ;$0F Secret dot's color (Grey)
       .byte $CB           ;$10 Challise's color (Flash)
       .byte $00           ;$11 Magnet's color (Black)
       .byte $00           ;$12 Null object's color (Black)

Store8: ;note: bits 4/5 set missile width
       .byte $17           ;$00 Invisible Surround's width (quadruple)
       .byte $10           ;$01 Yellow castle gate's width (single)
       .byte $10           ;$02 White castle gate's width (single)
       .byte $10           ;$03 Black castle gate's width (single)
       .byte $10           ;$04 Author signature's width (single)
       .byte $10           ;$05 Game selection digit's width (single)
       .byte $10           ;$06 Red Dragon's width (single)
       .byte $10           ;$07 Yellow Dragon's width (single)
       .byte $10           ;$08 Green Dragon's width (single)
       .byte $10           ;$09 Sword's width (single)
       .byte $17           ;$0A Bridge's width (quadruple)
       .byte $10           ;$0B Yellow key's width (single)
       .byte $10           ;$0C White key's width (single)
       .byte $10           ;$0D Black key's width (single)
       .byte $10           ;$0E Bat's width (single)
       .byte $10           ;$0F Secret dot's width (single)
       .byte $10           ;$10 Challise's width (single)
       .byte $10           ;$11 Magnet's width (single)
       .byte $10           ;$12 Null object's width (single)

;NOTE: All dragon matrices must be on the same page
;      If an object is listed before the dragon, it is "feared"
;      If an object is listed after the dragon, it is "guarded"
RedDragMatrix: ;Red Dragon Object Matrix
       .byte SwordR,RDragonR        ;Sword, Red Dragon
       .byte RDragonR,PlayerRoom    ;Red Dragon, Ball
       .byte RDragonR,ChalliseR     ;Red Dragon, Challise
       .byte RDragonR,WKeyR         ;Red Dragon, White Key
       .byte $00                    ;matrix end

YelDragMatrix: ;Yellow Dragon's Object Matrix
       .byte SwordR,YDragonR        ;Sword, Yellow Dragon
       .byte YKeyR,YDragonR         ;Yellow Key, Yellow Dragon
       .byte YDragonR,PlayerRoom    ;Yellow Dragon, Ball
       .byte YDragonR,ChalliseR     ;Yellow Dragon, Challise
NumberInfo: ;game selection digit location...
       .byte $00,$50,$38          ;#5 Number: Room 00,(50,40), first byte shared w/previous

GreenDragonMatrix: ;Green Dragon's Object Matrix
       .byte SwordR,GDragonR        ;Sword, Green Dragon
       .byte GDragonR,PlayerRoom    ;Green Dragon, Ball
       .byte GDragonR,ChalliseR     ;Green Dragon Challise
       .byte GDragonR,BridgeR       ;Green Dragon, Bridge
       .byte GDragonR,MagnetR       ;Green Dragon, Magnet
       .byte GDragonR,BKeyR         ;Green Dragon, Black Key
DragonStates: ;12 bytes, dragon's list of states
       .byte $00,<GfxDrag0,>GfxDrag0 ;State 00 (roaming), first byte shared w/previous
       .byte $01,<GfxDrag2,>GfxDrag2 ;State 01 (dead)
       .byte $02,<GfxDrag0,>GfxDrag0 ;State 02 (swallowed player)
       .byte $FF,<GfxDrag1,>GfxDrag1 ;State FF (biting player)

RoomDiffs: ;Room differences for different levels (level 1,2,3)
RD1:   ;<- These "RD" tags used in the room matrix...subtract RD0 to get proper $80 value
       .byte $10,$0F,$0F            ;Down from Room 01
RD2:
       .byte $05,$11,$11            ;Down from Room 02
RD3:
       .byte $1D,$0A,$0A            ;Down from Room 03
RD4:
       .byte $1C,$16,$16            ;U/L/R/D from Room 1B (Black Castle Room)
RD5:
       .byte $1B,$0C,$0C            ;Down from Room 1C
RD6:
       .byte $03,$0C,$0C            ;Up from Room 1D (Top Entry Room)

Loc_1: ;Room Bounds Data.
       .byte ChalliseR,$13,$1A      ;Challise, minimum screen number, maximum screen number
       .byte RDragonR,$01,$1D       ;Red Dragon
       .byte YDragonR,$01,$1D       ;Yellow Dragon
       .byte GDragonR,$01,$1D       ;Green Dragon
       .byte SwordR,$01,$1D         ;Sword
       .byte BridgeR,$01,$1D        ;Bridge
       .byte YKeyR,$01,$1D          ;Yellow Key
       .byte WKeyR,$01,$16          ;White Key
       .byte BKeyR,$01,$12          ;Black Key
       .byte BatR,$01,$1D           ;Bat
       .byte MagnetR,$01,$1D        ;Magnet
Loc_4:
       .word Game1Objects           ;object template for first game
       .word Game2Objects           ;object template for second
       .word Game2Objects           ;(reuse second for third before randomization)

EntryRoomOffsets: ;Castle Entry Rooms (Yellow, White, Black)...the 1st screen inside a castle
     .byte $12,$1A,$1B

       ORG  $2F00
       RORG $FF00

Y6pixels: ;used to select intial object pickup offset
       .byte 0,-6,6,0,0,-6,6,0,0,-6,6,0,0,-6,6 ;next byte shared
X6pixels: ;used to select intial object pickup offset
       .byte 0,0,0,0,6,6,6,6,-6,-6,-6,-6,0,0,0,0

RoomDataTable0: ;LSB of gfx ;room#, description
       .byte <Room00     ;$00 Number room
       .byte <Room01     ;$01 Below blue maze
       .byte <Room02     ;$02 Below yellow castle
       .byte <Room03     ;$03 Above the catacombs
       .byte <Room04     ;$04 Blue maze, below black castle
       .byte <Room05     ;$05 Blue maze, above entry
       .byte <Room06     ;$06 Blue maze bottom
       .byte <Room07     ;$07 Blue maze center
       .byte <Room08     ;$08 Blue maze entry
       .byte <Room09     ;$09 Catacombs middle
       .byte <Room0A     ;$0A Catacombs entry
       .byte <Room0B     ;$0B Catacombs side
       .byte <Room0C     ;$0C Right corridor, right of catacombs
       .byte <Room0D     ;$0D Left corridor, below white Castle
       .byte <Room0E     ;$0E Below left corridor
       .byte <Room0F     ;$0F White Castle (outside)
       .byte <Room10     ;$10 Black Castle (outside)
       .byte <Room11     ;$11 Yellow Castle (outside)
       .byte <Room12     ;$12 Inside yellow castle
       .byte <Room13     ;$13 Black maze, right of entry
       .byte <Room14     ;$14 Black maze, challise room
       .byte <Room15     ;$15 Black maze, dot's room
       .byte <Room16     ;$16 Black maze entry
       .byte <Room17     ;$17 Red maze, above black key's room
       .byte <Room18     ;$18 Red maze, above entry
       .byte <Room19     ;$19 Red maze, black key's room
       .byte <Room1A     ;$1A Red maze entry
       .byte <Room1B     ;$1B Just inside black castle
       .byte <Room1C     ;$1C Above right corridor, right of catacombs
       .byte <Room1D     ;$1D Under right corridor
       .byte <Room1E     ;$1E Warren Robinett's room

RoomDataTable1: ;MSB of gfx ;room#, description
;           Bit 5   (+$20): True if surround object needed
;           Bit 6/7 unused
       .byte >Room00     ;$00 Number room
       .byte >Room01     ;$01 Below blue maze
       .byte >Room02     ;$02 Below yellow castle
       .byte >Room03     ;$03 Above the catacombs
       .byte >Room04     ;$04 Blue maze, below black castle
       .byte >Room05     ;$05 Blue maze, above entry
       .byte >Room06     ;$06 Blue maze bottom
       .byte >Room07     ;$07 Blue maze center
       .byte >Room08     ;$08 Blue maze entry
       .byte >Room09+$20 ;$09 Catacombs middle
       .byte >Room0A+$20 ;$0A Catacombs entry
       .byte >Room0B+$20 ;$0B Catacombs side
       .byte >Room0C     ;$0C Right corridor, right of catacombs
       .byte >Room0D     ;$0D Left corridor, below white Castle
       .byte >Room0E     ;$0E Below left corridor
       .byte >Room0F     ;$0F White Castle (outside)
       .byte >Room10     ;$10 Black Castle (outside)
       .byte >Room11     ;$11 Yellow Castle (outside)
       .byte >Room12     ;$12 Inside yellow castle
       .byte >Room13+$20 ;$13 Black maze, right of entry
       .byte >Room14+$20 ;$14 Black maze, challise room
       .byte >Room15+$20 ;$15 Black maze, dot's room
       .byte >Room16+$20 ;$16 Black maze entry
       .byte >Room17     ;$17 Red maze, above black key's room
       .byte >Room18     ;$18 Red maze, above entry
       .byte >Room19     ;$19 Red maze, black key's room
       .byte >Room1A     ;$1A Red maze entry
       .byte >Room1B     ;$1B Just inside black castle
       .byte >Room1C     ;$1C Above right corridor, right of catacombs
       .byte >Room1D     ;$1D Under right corridor
       .byte >Room1E     ;$1E Warren Robinett's room

;Offset 2 : Color...if $08, invisible surround present...if an odd number, room is to Flash
RoomDataTable2: ;playfield color ;room#, description
       .byte $66     ;$00 Number room
       .byte $D8     ;$01 Below blue maze
       .byte $C8     ;$02 Below yellow castle
       .byte $E8     ;$03 Above the catacombs
       .byte $86     ;$04 Blue maze, below black castle
       .byte $86     ;$05 Blue maze, above entry
       .byte $86     ;$06 Blue maze bottom
       .byte $86     ;$07 Blue maze center
       .byte $86     ;$08 Blue maze entry
       .byte $08     ;$09 Catacombs middle
       .byte $08     ;$0A Catacombs entry
       .byte $08     ;$0B Catacombs side
       .byte $98     ;$0C Right corridor, right of catacombs
       .byte $B8     ;$0D Left corridor, below white Castle
       .byte $A8     ;$0E Below left corridor
       .byte $0C     ;$0F White Castle (outside)
       .byte $00     ;$10 Black Castle (outside)
       .byte $1A     ;$11 Yellow Castle (outside)
       .byte $1A     ;$12 Inside yellow castle
       .byte $08     ;$13 Black maze, right of entry
       .byte $08     ;$14 Black maze, challise room
       .byte $08     ;$15 Black maze, dot's room
       .byte $08     ;$16 Black maze entry
       .byte $36     ;$17 Red maze, above black key's room
       .byte $36     ;$18 Red maze, above entry
       .byte $36     ;$19 Red maze, black key's room
       .byte $36     ;$1A Red maze entry
       .byte $36     ;$1B Just inside black castle
       .byte $66     ;$1C Above right corridor, right of catacombs
       .byte $36     ;$1D Under right corridor
       .byte $66     ;$1E Warren Robinett's room

RoomDataTable4: ;room to the North ;room#, description
       .byte $00     ;$00 Number room
       .byte $08     ;$01 Below blue maze
       .byte $11     ;$02 Below yellow castle
       .byte $06     ;$03 Above the catacombs
       .byte $10     ;$04 Blue maze, below black castle
       .byte $1D     ;$05 Blue maze, above entry
       .byte $07     ;$06 Blue maze bottom
       .byte $04     ;$07 Blue maze center
       .byte $05     ;$08 Blue maze entry
       .byte $0A     ;$09 Catacombs middle
       .byte $03     ;$0A Catacombs entry
       .byte $09     ;$0B Catacombs side
       .byte $1C     ;$0C Right corridor, right of catacombs
       .byte $0F     ;$0D Left corridor, below white Castle
       .byte $0D     ;$0E Below left corridor
       .byte $0E     ;$0F White Castle (outside)
       .byte $01     ;$10 Black Castle (outside)
       .byte $06     ;$11 Yellow Castle (outside)
       .byte $12     ;$12 Inside yellow castle
       .byte $15     ;$13 Black maze, right of entry
       .byte $16     ;$14 Black maze, challise room
       .byte $13     ;$15 Black maze, dot's room
       .byte $14     ;$16 Black maze entry
       .byte $19     ;$17 Red maze, above black key's room
       .byte $1A     ;$18 Red maze, above entry
       .byte $17     ;$19 Red maze, black key's room
       .byte $18     ;$1A Red maze entry
       .byte RD4-RD0 ;$1B inside black castle
       .byte $1D     ;$1C Above right corridor, right of catacombs
       .byte RD6-RD0 ;$1D Under right corridor
       .byte $06     ;$1E Warren Robinett's room

RoomDataTable5: ;room to the Right ;room#, description
       .byte $00     ;$00 Number room
       .byte $02     ;$01 Below blue maze
       .byte $03     ;$02 Below yellow castle
       .byte $01     ;$03 Above the catacombs
       .byte $05     ;$04 Blue maze, below black castle
       .byte $06     ;$05 Blue maze, above entry
       .byte $04     ;$06 Blue maze bottom
       .byte $08     ;$07 Blue maze center
       .byte $07     ;$08 Blue maze entry
       .byte $0A     ;$09 Catacombs middle
       .byte $09     ;$0A Catacombs entry
       .byte $0C     ;$0B Catacombs side
       .byte $0D     ;$0C Right corridor, right of catacombs
       .byte $0B     ;$0D Left corridor, below white Castle
       .byte $10     ;$0E Below left corridor
       .byte $0F     ;$0F White Castle (outside)
       .byte $1C     ;$10 Black Castle (outside)
       .byte $03     ;$11 Yellow Castle (outside)
       .byte $12     ;$12 Inside yellow castle
       .byte $14     ;$13 Black maze, right of entry
       .byte $15     ;$14 Black maze, challise room
       .byte $16     ;$15 Black maze, dot's room
       .byte $13     ;$16 Black maze entry
       .byte $18     ;$17 Red maze, above black key's room
       .byte $17     ;$18 Red maze, above entry
       .byte $1A     ;$19 Red maze, black key's room
       .byte $19     ;$1A Red maze entry
       .byte RD4-RD0 ;$1B inside black castle
       .byte $07     ;$1C Above right corridor, right of catacombs
       .byte $01     ;$1D Under right corridor
       .byte $01     ;$1E Warren Robinett's room

RoomDataTable6: ;room to the South ;room#, description
       .byte $00     ;$00 Number room
       .byte RD1-RD0 ;$01 Below blue maze
       .byte RD2-RD0 ;$02 Below yellow castle
       .byte RD3-RD0 ;$03 Above the catacombs
       .byte $07     ;$04 Blue maze, below black castle
       .byte $08     ;$05 Blue maze, above entry
       .byte $03     ;$06 Blue maze bottom
       .byte $06     ;$07 Blue maze center
       .byte $01     ;$08 Blue maze entry
       .byte $0B     ;$09 Catacombs middle
       .byte $09     ;$0A Catacombs entry
       .byte $1C     ;$0B Catacombs side
       .byte $1D     ;$0C Right corridor, right of catacombs
       .byte $0E     ;$0D Left corridor, below white Castle
       .byte $0F     ;$0E Below left corridor
       .byte $0D     ;$0F White Castle (outside)
       .byte $04     ;$10 Black Castle (outside)
       .byte $02     ;$11 Yellow Castle (outside)
       .byte $12     ;$12 Inside yellow castle
       .byte $15     ;$13 Black maze, right of entry
       .byte $16     ;$14 Black maze, challise room
       .byte $13     ;$15 Black maze, dot's room
       .byte $1B     ;$16 Black maze entry
       .byte $19     ;$17 Red maze, above black key's room
       .byte $1A     ;$18 Red maze, above entry
       .byte $17     ;$19 Red maze, black key's room
       .byte $18     ;$1A Red maze entry
       .byte RD4-RD0 ;$1B inside black castle
       .byte RD5-RD0 ;$1C Above right corridor, right of catacombs
       .byte $10     ;$1D Under right corridor
       .byte $06     ;$1E Warren Robinett's room

RoomDataTable7: ;room to the Left ;room#, description
       .byte $00     ;$00 Number room
       .byte $03     ;$01 Below blue maze
       .byte $01     ;$02 Below yellow castle
       .byte $02     ;$03 Above the catacombs
       .byte $06     ;$04 Blue maze, below black castle
       .byte $04     ;$05 Blue maze, above entry
       .byte $05     ;$06 Blue maze bottom
       .byte $08     ;$07 Blue maze center
       .byte $07     ;$08 Blue maze entry
       .byte $0A     ;$09 Catacombs middle
       .byte $09     ;$0A Catacombs entry
       .byte $0D     ;$0B Catacombs side
       .byte $0B     ;$0C Right corridor, right of catacombs
       .byte $0C     ;$0D Left corridor, below white Castle
       .byte $10     ;$0E Below left corridor
       .byte $0F     ;$0F White Castle (outside)
       .byte $1C     ;$10 Black Castle (outside)
       .byte $01     ;$11 Yellow Castle (outside)
       .byte $12     ;$12 Inside yellow castle
       .byte $16     ;$13 Black maze, right of entry
       .byte $13     ;$14 Black maze, challise room
       .byte $14     ;$15 Black maze, dot's room
       .byte $15     ;$16 Black maze entry
       .byte $18     ;$17 Red maze, above black key's room
       .byte $17     ;$18 Red maze, above entry
       .byte $1A     ;$19 Red maze, black key's room
       .byte $19     ;$1A Red maze entry
       .byte RD4-RD0 ;$1B inside black castle
       .byte $08     ;$1C Above right corridor, right of catacombs
       .byte $03     ;$1D Under right corridor
       .byte $03     ;$1E Warren Robinett's room

       ORG  $2FF8
       RORG $FFF8
;do not use $fff8/$fff9...bankswitch hotspot locations
       .byte "2009"
       .word START2,START2
